﻿/// <summary>
/// Aquesta és la classe responsable de la permanencia de les dades, connecta a la base de dades Microsoft SQL vie Linq to Entities 
/// </summary>

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Biblioteca.BibliotecaServiceEntities;
using System.Data.Objects;
//using System.Security.Cryptography;
using System.Web.Helpers;
//Helpers.Crypto;

//using WebMatrix.Data;
//using WebMatrix.WebData;
//using System.Web;
//using Biblioteca.BibliotecaServiceDAL;

namespace Biblioteca.BibliotecaServiceDAL
{
    public class BibliotecaDAO
    {
        BibliotecaEntities2 Biblioentity = new BibliotecaEntities2();

        /// <summary>
        /// Obtenim l'identificador d'usuari a partir del nom d'usuari i el password
        /// </summary>
        /// <parm name="usuariI">Nom de l'usuari</param>
        /// <parm name="password">password de l'usuari</param>
        public int login(string usuariIn, string password)
        {
            IQueryable<UserProfile> perfil = from p in Biblioentity.UserProfiles
                                             where (string)p.UserName == usuariIn
                                             select p;

            UserProfile elPerfil = perfil.FirstOrDefault();
            if (elPerfil != null)
            {
                IQueryable<webpages_Membership> member = from p in Biblioentity.webpages_Membership
                                                 where (int)p.UserId == elPerfil.UserId
                                                 select p;
                webpages_Membership elMember = member.FirstOrDefault();
                if (elMember != null)
                {
                    // Calculem el hash del password introduït vs el hash del password de l'usuari
                    bool result = System.Web.Helpers.Crypto.VerifyHashedPassword(elMember.Password, password);
                    IQueryable<Usuari> usuari = from p in Biblioentity.Usuaris
                                                    where (int)p.UserId == elPerfil.UserId
                                                    select p;
                    Usuari lusuari = usuari.FirstOrDefault();
                    if (result)
                    {
                        if (lusuari == null)
                        {
                            return 0;
                        }
                        else
                        {
                            return lusuari.UsuariId;
                        }
                    }
                    else
                    {
                        return 0;
                    }
                }
            }
            return 0;
        }


        /// <summary>
        /// Obtenim una llista de llibres filtrats segons els criteris de búsqueda
        /// </summary>
        /// <parm name="titolSearch">La cadena de búsqueda en el camp titol</param>
        /// <parm name="descripcioSearch">La cadena de búsqueda en el camp descripcio</param>
        /// <parm name="AutorId">el identificador d'autor per filtrar per autor</param>
        /// <parm name="EditorialId">el identificador d'editorial per filtrar per editorial</param>
        /// <parm name="CategoriaId">el identificador d'autor per filtrar per categoria</param>
        /// <parm name="TipusLlibreId">el identificador de tipus de llibre per filtrar per tipus de llibre</param>
        public List<LlibreEntity> GetLlibres(string titolSearch, string descripcioSearch, string AutorId, string EditorialId, string CategoriaId, string TipusLlibreId)
        {
            var llistaLlibres = from Ll in Biblioentity.Llibres select Ll;
            List<LlibreEntity> llistaLlibresEntity = new List<LlibreEntity>();

            // Filtrem per títol
            if (!String.IsNullOrEmpty(titolSearch))
            {
                llistaLlibres = llistaLlibres.Where(l => l.Titol.ToUpper().Contains(titolSearch.ToUpper()));
            }

            //Filtrem per descripció
            if (!String.IsNullOrEmpty(descripcioSearch))
            {
                llistaLlibres = llistaLlibres.Where(l => l.Descripcio.ToUpper().Contains(descripcioSearch.ToUpper()));
            }

            //Filtrem per autor
            int currentAutor;
            if (!String.IsNullOrEmpty(AutorId))
            {
                currentAutor = Convert.ToInt32(AutorId);
                llistaLlibres = llistaLlibres.Where(l => l.AutorId == currentAutor);
            }

            //Filtrem per Editorial
            int currentEditorial;
            if (!String.IsNullOrEmpty(EditorialId))
            {
                currentEditorial = Convert.ToInt32(EditorialId);
                llistaLlibres = llistaLlibres.Where(l => l.EditorialId == currentEditorial);
            }

            //Filtrem per Categoria
            int currentCategoria;
            if (!String.IsNullOrEmpty(CategoriaId))
            {
                currentCategoria = Convert.ToInt32(CategoriaId);
                llistaLlibres = llistaLlibres.Where(l => l.CategoriaId == currentCategoria);
            }

            //Filtrem per TipusLlibre
            int currentTipusLlibre;
            if (!String.IsNullOrEmpty(TipusLlibreId))
            {
                currentTipusLlibre = Convert.ToInt32(TipusLlibreId);
                llistaLlibres = llistaLlibres.Where(l => l.TipusLlibreId == currentTipusLlibre);
            }

            
            // Convertim els llibres de la llista d'elements llibre en una llista d'elements llibreEntity
            LlibreEntity nouLlibre = null;
            foreach (Llibre Ll in llistaLlibres)
            {
                nouLlibre = new LlibreEntity();
                this.ConvertirLlibreContractAEntityLlibre(nouLlibre, Ll);
                llistaLlibresEntity.Add(nouLlibre);
            }
            Biblioentity.Dispose();
            return llistaLlibresEntity;
        }

        /// <summary>
        /// Retorna el llibre i el retornem convertit en llibreEntity
        /// </summary>
        /// <parm name="llibreId">l'identificador del llibre que busquem</param>
        public LlibreEntity GetLlibre(int llibreId)
        {
            IQueryable <Llibre> llibres = from ll in Biblioentity.Llibres 
                         where ll.LlibreId == llibreId select ll;

            Llibre llibre = llibres.First();
            LlibreEntity nouLl = new LlibreEntity();

            ConvertirLlibreContractAEntityLlibre(nouLl, llibre);
            return nouLl;
        }

        /// <summary>
        /// Retorna Els prestecs de l'usuari passat com a paràmetre,
        /// retornem la llista de prestecs covertits a prestecEntity
        /// </summary>
        /// <parm name="userId">L'usuari Id per generar la llista de 
        /// prèstecs, userId the la web externa</param>
        public List<PrestecEntity> GetPrestecs(int userId)
        {
            // Inicialitzem el model
            BibliotecaEntities2 Biblioentity = new BibliotecaEntities2();

            // Obtenim l'usuari
            IQueryable<Usuari> usuaris = from ll in Biblioentity.Usuaris
                                         where ll.UserId == userId
                                         select ll;
            Usuari usuari = usuaris.First();

            // Obtenim el llistat de préstecs de l'usuari
            var llistaPrestecs = from P in Biblioentity.Prestecs select P;
            List<PrestecEntity> llistaPrestecsEntity = new List<PrestecEntity>();
            llistaPrestecs = llistaPrestecs.Where(p => p.UsuariId == usuari.UsuariId);
            PrestecEntity nouPrestec = null;

            // Convertim la llista de préstecs a una llista préstecs Entity
            foreach (Prestec p in llistaPrestecs)
            {
                nouPrestec = new PrestecEntity();
                this.ConvertirPrestecContractAEntityPrestec(nouPrestec, p);
                llistaPrestecsEntity.Add(nouPrestec);
            }
            Biblioentity.Dispose();
            return llistaPrestecsEntity;
        }

        /// <summary>
        /// Retorna Els prestecs de l'usuari passat com a paràmetre, retornem la llista de prestecs covertits a prestecEntity
        /// </summary>
        /// <parm name="usuariId">L'usuari Id per generar la llista de prèstecs, l'identificador d'usuari intern</param>
        public List<PrestecEntity> GetPrestecsByUsuariId(int usuariId)
        {
            System.IO.StreamWriter file = new System.IO.StreamWriter("c:\\projects\\test2.txt");
            BibliotecaEntities2 Biblioentity = new BibliotecaEntities2();
            IQueryable<Usuari> usuaris = from ll in Biblioentity.Usuaris
                                         where ll.UsuariId == usuariId
                                         select ll;

            int quants = usuaris.Count();
            file.WriteLine("Usuaris amb el userid: " + usuaris.Count().ToString());

            Usuari usuari = usuaris.First();

            var llistaPrestecs = from P in Biblioentity.Prestecs select P;
            List<PrestecEntity> llistaPrestecsEntity = new List<PrestecEntity>();
            llistaPrestecs = llistaPrestecs.Where(p => p.UsuariId == usuari.UsuariId);
            PrestecEntity nouPrestec = null;

            quants = llistaPrestecs.Count();

            file.WriteLine("Prestecs: " + llistaPrestecs.Count().ToString());
            foreach (Prestec p in llistaPrestecs)
            {
                nouPrestec = new PrestecEntity();
                this.ConvertirPrestecContractAEntityPrestec(nouPrestec, p);
                llistaPrestecsEntity.Add(nouPrestec);
            }
            Biblioentity.Dispose();

            // Write the string to a file.
            


            file.Close();
            return llistaPrestecsEntity;
        }

        /// <summary>
        /// Retorna la llista d'autors convertits a AutorEntity
        /// </summary>
        public List<AutorEntity> GetAutors()
        {
            var llistaAutors = from Ll in Biblioentity.Autors select Ll;
            List<AutorEntity> llistaAutorsEntity = new List<AutorEntity>();

            AutorEntity nouAutor = null;
            foreach (Autor Ll in llistaAutors)
            {
                nouAutor = new AutorEntity();
                this.ConvertirAutorContractAEntityAutor(nouAutor, Ll);
                llistaAutorsEntity.Add(nouAutor);
            }
            Biblioentity.Dispose();
            return llistaAutorsEntity;
        }

        /// <summary>
        /// Retorna una llista d'editorials convertides a EditorialEnity
        /// </summary>
        public List<EditorialEntity> GetEditorials()
        {
            var llistaEditorials = from Ll in Biblioentity.Editorials select Ll;
            List<EditorialEntity> llistaEditorialsEntity = new List<EditorialEntity>();

            EditorialEntity nouEditorial = null;
            foreach (Editorial Ll in llistaEditorials)
            {
                nouEditorial = new EditorialEntity();
                this.ConvertirEditorialContractAEntityEditorial(nouEditorial, Ll);
                llistaEditorialsEntity.Add(nouEditorial);
            }
            Biblioentity.Dispose();
            return llistaEditorialsEntity;
        }

        /// <summary>
        /// Retorna una llista de categories convertides a CategorialEnity
        /// </summary>
        public List<CategoriaEntity> GetCategories()
        {
            var llistaCategorias = from Ll in Biblioentity.Categorias select Ll;
            List<CategoriaEntity> llistaCategoriasEntity = new List<CategoriaEntity>();

            CategoriaEntity nouCategoria = null;
            foreach (Categoria Ll in llistaCategorias)
            {
                nouCategoria = new CategoriaEntity();
                this.ConvertirCategoriaContractAEntityCategoria(nouCategoria, Ll);
                llistaCategoriasEntity.Add(nouCategoria);
            }
            Biblioentity.Dispose();
            return llistaCategoriasEntity;
        }

        /// <summary>
        /// Retorna una llista de tipus de llibre convertits a TipusLibreEnity
        /// </summary>
        public List<TipusLlibreEntity> GetTipusLlibres()
        {
            var llistaTipusLlibres = from Ll in Biblioentity.TipusLlibres select Ll;
            List<TipusLlibreEntity> llistaTipusLlibresEntity = new List<TipusLlibreEntity>();

            TipusLlibreEntity nouTipusLlibre = null;
            foreach (TipusLlibre Ll in llistaTipusLlibres)
            {
                nouTipusLlibre = new TipusLlibreEntity();
                this.ConvertirTipusLlibreContractAEntityTipusLlibre(nouTipusLlibre, Ll);
                llistaTipusLlibresEntity.Add(nouTipusLlibre);
            }
            Biblioentity.Dispose();
            return llistaTipusLlibresEntity;
        }

        /// <summary>
        /// Retorna les configuracions globals convertides a ConfiguracioEnity
        /// </summary>
        public ConfiguracioEntity GetConfiguracio()
        {
            IQueryable<Configuracio> Configuracios = from ll in Biblioentity.Configuracios
                                         where ll.ConfiguracioId == 1
                                         select ll;

            Configuracio configuracio = Configuracios.First();
            ConfiguracioEntity nouU = new ConfiguracioEntity();

            ConvertirConfiguracioContractAEntityConfiguracio(nouU, configuracio);
            return nouU;
        }


        /// <summary>
        /// Retorna l'usuari en format UsuariEntity
        /// </summary>
        /// <parm name="usuariId">el identificador de l'usuari</param>
        public UsuariEntity GetUsuaribyUsuariID(int usuariId)
        {
            IQueryable<Usuari> usuaris = from ll in Biblioentity.Usuaris
                                         where ll.UsuariId == usuariId
                                         select ll;

            Usuari usuari = usuaris.FirstOrDefault();
            UsuariEntity nouU = new UsuariEntity();

            ConvertirUsuariContractAEntityUsuari(nouU, usuari);
            return nouU;
        }

        /// <summary>
        /// Retorna l'usuari en format UsuariEntity
        /// </summary>
        /// <parm name="userId">el identificador de l'usuari al perfil web</param>
        public UsuariEntity GetUsuari(int userId)
        {
            IQueryable<Usuari> usuaris = from ll in Biblioentity.Usuaris
                                         where ll.UserId == userId
                                         select ll;

            Usuari usuari = usuaris.FirstOrDefault();
            UsuariEntity nouU = new UsuariEntity();

            ConvertirUsuariContractAEntityUsuari(nouU, usuari);
            return nouU;
        }

        /// <summary>
        /// Actualitza les dades de l'usuari passat com a peràmetre, previament convetex l'usuari a Usuari 
        /// (model de la base de dades) des de UsuariEntity
        /// </summary>
        /// <parm name="usuariEnt">l'usuari en format UsuariEntity</param>
        public string UpdateUsuari(UsuariEntity usuariEnt)
        {
            // Busquem l'usuari
            Usuari usuari = (from u in Biblioentity.Usuaris where u.UsuariId == usuariEnt.UsuariId select u).FirstOrDefault();
            if (usuari == null)
            {
                // Si no el trobem retornem missatge d'error
                return ("L'usuari no existeix");
            }
            // Convertim l'usuari Entity a usuari del model de la base de dades
            usuari.Nom = usuariEnt.Nom;
            usuari.Cognom = usuariEnt.Cognom;
            usuari.Cognom2 = usuariEnt.Cognom2;
            usuari.DNI = usuariEnt.DNI;
            usuari.Direccio = usuariEnt.Direccio;
            usuari.Poblacio = usuariEnt.Poblacio;
            usuari.CodiPostal = usuariEnt.CodiPostal;
            usuari.Email = usuariEnt.Email;
            usuari.Notificacions = (bool)usuariEnt.Notificacions;
            usuari.AccesWeb = (bool)usuariEnt.AccesWeb;
            try
            {
                // Actualitzem la base de dades
                Biblioentity.SaveChanges();
                Biblioentity.Dispose();
            }
            catch
            {
                // Si l'actualització no és possible retornem missatge d'error
                return ("Error actualitzant l'usuari");
            }
            return ("");
        }

        /// <summary>
        /// Converteix un llibre (model de la base de dades) en llibreEntity
        /// </summary>
        /// <parm name="llibreEntity">Llibre a convertir</param>
        /// <parm name="llibre">llibreEntity convertit</param>
        private void ConvertirLlibreContractAEntityLlibre(LlibreEntity llibreEntity, Llibre llibre)
        {
            llibreEntity.LlibreId = llibre.LlibreId;
            llibreEntity.ISBN = llibre.ISBN;
            llibreEntity.Titol = llibre.Titol;
            llibreEntity.Descripcio = llibre.Descripcio;
            llibreEntity.CategoriaId = llibre.CategoriaId;
            llibreEntity.TipusLlibreId = llibre.TipusLlibreId;
            llibreEntity.AutorId = llibre.AutorId;
            llibreEntity.EnPrestec = llibre.EnPrestec;
            llibreEntity.Autor = llibre.Autor.NomAutor;
            llibreEntity.Categoria = llibre.Categoria.Nom;
            llibreEntity.TipusLlibre = llibre.TipusLlibre.Nom;
            llibreEntity.Editorial = llibre.Editorial.Nom;
        }

        /// <summary>
        /// Converteix un Prestec (model de la base de dades) en prestecEntity
        /// </summary>
        /// <parm name="prestecEntity">Prestec a convertir</param>
        /// <parm name="llibre">PrestecEntity convertit</param>
        private void ConvertirPrestecContractAEntityPrestec(PrestecEntity prestecEntity, Prestec prestec)
        {
            prestecEntity.PrestecId = prestec.PrestecId;
            prestecEntity.LlibreId = prestec.LlibreId;
            prestecEntity.UsuariId = prestec.UsuariId;
            prestecEntity.DataPrestec = prestec.dataPrestec;
            prestecEntity.DataEstimada = prestec.dataEstimada;
            prestecEntity.Nota = prestec.nota;
            prestecEntity.EnPrestec = (bool)prestec.EnPrestec;
            prestecEntity.TitolLlibre = prestec.Llibre.Titol;
        }

        /// <summary>
        /// Converteix un Autor (model de la base de dades) en AutorEntity
        /// </summary>
        /// <parm name="autorEntity">Autor a convertir</param>
        /// <parm name="autor">AutorEntity convertit</param>
        private void ConvertirAutorContractAEntityAutor(AutorEntity autorEntity, Autor autor)
        {
            autorEntity.AutorId = autor.AutorId;
            autorEntity.NomAutor = autor.NomAutor;
        }

        /// <summary>
        /// Converteix un Usuari (model de la base de dades) en UsuariEntity
        /// </summary>
        /// <parm name="usuariEntity">Usuari a convertir</param>
        /// <parm name="usuari">UsuariEntity convertit</param>
        private void ConvertirUsuariContractAEntityUsuari(UsuariEntity usuariEntity, Usuari usuari)
        {
            usuariEntity.UsuariId = usuari.UsuariId;
            usuariEntity.Nom = usuari.Nom;
            usuariEntity.Cognom = usuari.Cognom;
            usuariEntity.Cognom2 = usuari.Cognom2;
            usuariEntity.DNI = usuari.DNI;
            usuariEntity.Direccio = usuari.Direccio;
            usuariEntity.Poblacio = usuari.Poblacio;
            usuariEntity.CodiPostal = usuari.CodiPostal;
            usuariEntity.Email = usuari.Email;
            usuariEntity.Notificacions = (bool)usuari.Notificacions;
            usuariEntity.AccesWeb = (bool)usuari.AccesWeb;
        }

        /// <summary>
        /// Converteix una Editorial (model de la base de dades) en EditorialEntity
        /// </summary>
        /// <parm name="editorialEntity">Editorial a convertir</param>
        /// <parm name="editorial">EditorialEntity convertit</param>
        private void ConvertirEditorialContractAEntityEditorial(EditorialEntity editorialEntity, Editorial editorial)
        {
            editorialEntity.EditorialId = editorial.EditorialId;
            editorialEntity.Nom = editorial.Nom;
            editorialEntity.Adreca = editorial.Adreca;
            editorialEntity.CodiPostal = editorial.CodiPostal;
            editorialEntity.Tfn = editorial.Tfn;
            editorialEntity.Ciutat = editorial.Ciutat;
        }

        /// <summary>
        /// Converteix una Categoria (model de la base de dades) en CategoriaEntity
        /// </summary>
        /// <parm name="categoriaEntity">Categoria a convertir</param>
        /// <parm name="categoria">CategoriaEntity convertit</param>
        private void ConvertirCategoriaContractAEntityCategoria(CategoriaEntity categoriaEntity, Categoria categoria)
        {
            categoriaEntity.CategoriaId = categoria.CategoriaId;
            categoriaEntity.Nom= categoria.Nom;
            categoriaEntity.Descripcio = categoria.Descripcio;
        }

        /// <summary>
        /// Converteix un TipusLlibre (model de la base de dades) en TipusLlibreEntity
        /// </summary>
        /// <parm name="tipusLlibreEntity">TipusLlibreEntity a convertir</param>
        /// <parm name="tipusLlibre">TipusLlibreEntity convertit</param>
        private void ConvertirTipusLlibreContractAEntityTipusLlibre(TipusLlibreEntity tipusLlibreEntity, TipusLlibre tipusLlibre)
        {
            tipusLlibreEntity.TipusLlibreId = tipusLlibre.TipusLlibreId;
            tipusLlibreEntity.Nom = tipusLlibre.Nom;
        }

        /// <summary>
        /// Converteix una Confguracio (model de la base de dades) en ConfiguracioEntity
        /// </summary>
        /// <parm name="configuracioEntity">Configuracio a convertir</param>
        /// <parm name="configuracio">ConfiguracioEntity convertit</param>
        private void ConvertirConfiguracioContractAEntityConfiguracio(ConfiguracioEntity configuracioEntity, Configuracio configuracio)
        {
            configuracioEntity.ConfiguracioId = configuracio.ConfiguracioId;
            configuracioEntity.DiesPrestec = (int)configuracio.DiesPrestec;
            configuracioEntity.NomBiblioteca = configuracio.NomBiblioteca;
            configuracioEntity.EmailContacte = configuracio.EmailContacte;
            configuracioEntity.Emailnotificacions = configuracio.Emailnotificacions;
            configuracioEntity.ServidorSMTP = configuracio.ServidorSMTP;
            configuracioEntity.PasswordSMTP = configuracio.PasswordSMTP;
            configuracioEntity.NotificacionsActives = (bool)configuracio.NotificacionsActives;
            configuracioEntity.UsuariSMTP = configuracio.UsuariSMTP;
            configuracioEntity.PortSMTP = (int)configuracio.PortSMTP;
        }
    }
}
