﻿/// <summary>
/// Aquest és la classe del servei WCF
/// </summary>

using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using Biblioteca.BibliotecaServiceEntities;
using Biblioteca.BibliotecaServiceLogica;

namespace Biblioteca.BibliotecaServices
{
    public class BibliotecaService : IBibliotecaService
    {
        /// <summary>
        /// Obtenim l'identificador de l'usuari a partir del nom d'usuari i el password
        /// </summary>
        /// <parm name="usuari">Nom d'usuari</param>
        /// <parm name="password">PAssword</param>
        public int Login(string usuari, string password)
        {
            BibliotecaLogica BiblioL = new BibliotecaLogica();
            return BiblioL.Login(usuari, password);
        }
        /// <summary>
        /// Obtenim una llista de llibres filtrats segons els criteris de búsqueda
        /// </summary>
        /// <parm name="titolSearch">La cadena de búsqueda en el camp titol</param>
        /// <parm name="descripcioSearch">La cadena de búsqueda en el camp descripcio</param>
        /// <parm name="AutorId">el identificador d'autor per filtrar per autor</param>
        /// <parm name="EditorialId">el identificador d'editorial per filtrar per editorial</param>
        /// <parm name="CategoriaId">el identificador d'autor per filtrar per categoria</param>
        /// <parm name="TipusLlibreId">el identificador de tipus de llibre per filtrar per tipus de llibre</param>
        public IList<Llibre> GetLlibres(string titolSearch, string descripcioSearch, string AutorId, string EditorialId, string CategoriaId, string TipusLlibreId)
        {
            List<Llibre> llibres = new List<Llibre>();
            List<LlibreEntity> llibresE = new List<LlibreEntity>();
            BibliotecaLogica BiblioL = new BibliotecaLogica();
            Llibre llibreActual;
            // Obtenim els llibres de la capa de negoci
            llibresE = BiblioL.GetLlibres(titolSearch, descripcioSearch, AutorId, EditorialId, CategoriaId, TipusLlibreId);
            // Convertim els llibres Entity en el model definit per la interficie del servei
            foreach (LlibreEntity LlE in llibresE)
            {
                llibreActual = new Llibre();
                ConvertirEntityLlibreALlibreContractData(LlE, llibreActual);
                llibres.Add(llibreActual);
            }
            llibres.OrderBy(Ll => Ll.Titol);
            return llibres;
        }

        /// <summary>
        /// Obtenim el llibre que té per identificador llibreId
        /// </summary>
        /// <parm name="llibreId">Identificador del llibre</param>
        public Llibre GetLlibre(int llibreId)
        {
            Llibre llibre = new Llibre();
            LlibreEntity llibreE = new LlibreEntity();
            BibliotecaLogica BiblioL = new BibliotecaLogica();

            llibreE = BiblioL.GetLlibre(llibreId);
            ConvertirEntityLlibreALlibreContractData(llibreE, llibre);
            return llibre;
        }


        /// <summary>
        /// Obtenim els prèstecs d'un usuari
        /// </summary>
        /// <parm name="usuariId">Identificador de l'usuari</param>
        public IList<Prestec> GetPrestecs(int usuariId)
        {
            List<Prestec> prestecs = new List<Prestec>();
            List<PrestecEntity> PrestecsE = new List<PrestecEntity>();
            BibliotecaLogica BiblioL = new BibliotecaLogica();
            Prestec prestecActual;
            PrestecsE = BiblioL.GetPrestecs(usuariId);
            foreach (PrestecEntity PE in PrestecsE)
            {
                prestecActual = new Prestec();
                ConvertirEntityPrestecAPrestecContractData(PE, prestecActual);
                prestecs.Add(prestecActual);
            }
            return prestecs;
        }

        /// <summary>
        /// Obtenim les dades d'un usuari
        /// </summary>
        /// <parm name="usuariId">Identificador de l'usuari</param>
        public Usuari GetUsuari(int UsuariId)
        {
            Usuari Usuari = new Usuari();
            UsuariEntity UsuariE = new UsuariEntity();
            BibliotecaLogica BiblioL = new BibliotecaLogica();

            UsuariE = BiblioL.GetUsuari(UsuariId);
            ConvertirEntityUsuariToUsuariContractData(UsuariE, Usuari);
            return Usuari;
        }

        /// <summary>
        /// Obtenim un llistat d'autors
        /// </summary>
        public IList<Autor> GetAutors()
        {
            List<Autor> autors = new List<Autor>();
            List<AutorEntity> autorsE = new List<AutorEntity>();
            BibliotecaLogica BiblioL = new BibliotecaLogica();
            Autor autorActual;
            autorsE = BiblioL.GetAutors();
            foreach (AutorEntity LlE in autorsE)
            {
                autorActual = new Autor();
                ConvertirAutorEntityToAutorContractData(LlE, autorActual);
                autors.Add(autorActual);
            }
            autors.OrderBy(Ll => Ll.NomAutor);
            return autors;
        }

        /// <summary>
        /// Actualitzem l'usuari
        /// </summary>
        /// <parm name="usuariId">Identificador de l'usuari</param>
        public string UpdateUsuari(Usuari usuari)
        {
            BibliotecaLogica BiblioL = new BibliotecaLogica();
            UsuariEntity usuariEnt = new UsuariEntity();
            ConvertirUsuariContractDataToUsuariEntity(usuariEnt, usuari);
            try
            {
                BiblioL.UpdateUsuari(usuariEnt);
                return String.Empty;
            }
            catch (Exception ex)
            {
                //throw;
                return ex.Message;
                //throw new FaultException<ErrorUsuariDataIncorrecta>(new ErrorUsuariDataIncorrecta(ex.Message.ToString()), "Product Fault");
            }
        }

        /// <summary>
        /// Obtenim la configuració
        /// </summary>
        public Configuracio GetConfiguracio()
        {
            Configuracio Configuracio = new Configuracio();
            ConfiguracioEntity ConfiguracioE = new ConfiguracioEntity();
            BibliotecaLogica BiblioL = new BibliotecaLogica();

            ConfiguracioE = BiblioL.GetConfiguracio();
            ConvertirEntityConfiguracioToConfiguracioContractData(ConfiguracioE, Configuracio);
            return Configuracio;
        }

        /// <summary>
        /// Obtenim un llistat d'editorials
        /// </summary>
        public IList<Editorial> GetEditorials()
        {
            List<Editorial> Editorials = new List<Editorial>();
            List<EditorialEntity> EditorialsE = new List<EditorialEntity>();
            BibliotecaLogica BiblioL = new BibliotecaLogica();
            Editorial EditorialActual;
            EditorialsE = BiblioL.GetEditorials();
            foreach (EditorialEntity LlE in EditorialsE)
            {
                EditorialActual = new Editorial();
                ConvertirEditorialEntityToEditorialContractData(LlE, EditorialActual);
                Editorials.Add(EditorialActual);
            }
            Editorials.OrderBy(Ll => Ll.Nom);
            return Editorials;
        }

        /// <summary>
        /// Obtenim un llistat de categories
        /// </summary>
        public IList<Categoria> GetCategories()
        {
            List<Categoria> Categorias = new List<Categoria>();
            List<CategoriaEntity> CategoriasE = new List<CategoriaEntity>();
            BibliotecaLogica BiblioL = new BibliotecaLogica();
            Categoria CategoriaActual;
            CategoriasE = BiblioL.GetCategories();
            foreach (CategoriaEntity LlE in CategoriasE)
            {
                CategoriaActual = new Categoria();
                ConvertirCategoriaEntityToCategoriaContractData(LlE, CategoriaActual);
                Categorias.Add(CategoriaActual);
            }
            Categorias.OrderBy(Ll => Ll.Nom);
            return Categorias;
        }

        /// <summary>
        /// Obtenim un llistat de Tipus de llibre
        /// </summary>
        public IList<TipusLlibre> GetTipusLlibre()
        {
            List<TipusLlibre> TipusLlibres = new List<TipusLlibre>();
            List<TipusLlibreEntity> TipusLlibresE = new List<TipusLlibreEntity>();
            BibliotecaLogica BiblioL = new BibliotecaLogica();
            TipusLlibre TipusLlibreActual;
            TipusLlibresE = BiblioL.GetTipusLlibre();
            foreach (TipusLlibreEntity LlE in TipusLlibresE)
            {
                TipusLlibreActual = new TipusLlibre();
                ConvertirTipusLlibreEntityToTipusLlibreContractData(LlE, TipusLlibreActual);
                TipusLlibres.Add(TipusLlibreActual);
            }
            TipusLlibres.OrderBy(Ll => Ll.Nom);
            return TipusLlibres;
        }

     /*   /// <summary>
        /// Obtenim les dades d'un usuari a partir del seu nom d'usuari i password
        /// </summary>
        /// <parm name="usuari">Nom d'usuari</param>
        /// <parm name="password">PAssword</param>
        public Usuari GetUsuariPhone(string usuari, string password)
        {
            int usuariId = Login (usuari, password);
            Usuari usuariResult = null;
            if (usuariId == 0)
            {
                return null;
            }
            else
            {
                usuariResult = GetUsuari(usuariId);
                return usuariResult;
            }
        }*/

        /// <summary>
        /// Obtenim les dades d'un usuari a partir del seu nom d'usuari i password
        /// </summary>
        /// <parm name="usuari">Nom d'usuari</param>
        /// <parm name="password">PAssword</param>
        public Usuari GetUsuariPhonev2(string usuari, string password)
        {
            BibliotecaLogica BiblioL = new BibliotecaLogica();
            int usuariId = BiblioL.Login(usuari, password);
            Usuari Usuari = new Usuari();
            UsuariEntity UsuariE = new UsuariEntity();
            if (usuariId == 0)
            {
                return null;
            }
            try
            {
                UsuariE = BiblioL.GetUsuaribyUsuariId(usuariId);
            }
            catch
            {
                return null;
            }

            try
            {
                ConvertirEntityUsuariToUsuariContractData(UsuariE, Usuari);
            }
            catch
            {
                return null;
            }
            return Usuari;
        }

      /*  /// <summary>
        /// Actualitzem l'usuari
        /// </summary>
        /// <parm name="usuari">l'informació de l'usuari a actualitzar</param>
        /// <parm name="usuari">Nom d'usuari</param>
        /// <parm name="password">Password</param>
        public String UpdateUsuariPhone(string nomUsuari, string password, Usuari usuari)
        {
            int usuariId = Login(nomUsuari, password);
            if (usuariId != 0)
            {
                return UpdateUsuari (usuari);
            }
            else
            {
                return ("Nom d'usuari o password incorrectes");
            }
        }
        */

        /// <summary>
        /// Obtenim els prèstecs d'un usuari a partir del nom d'usuari i el password
        /// </summary>
        /// <parm name="usuari">Nom d'usuari</param>
        /// <parm name="password">Password</param>
        public IList<Prestec> GetPrestecsPhone(string usuari, string password)
        {
            BibliotecaLogica BiblioL = new BibliotecaLogica();
            List<PrestecEntity> PrestecsE = new List<PrestecEntity>();
            List<Prestec> elsPrestecs = new List<Prestec>();
            int usuariId = BiblioL.Login(usuari, password);
            //Usuari Usuari = new Usuari();
            //UsuariEntity UsuariE = new UsuariEntity();
            if (usuariId == 0)
            {
                return null;
            }
            try
            {
                PrestecsE = BiblioL.GetPrestecsByUsuariId(usuariId);
                Prestec prestecActual;
                foreach (PrestecEntity PE in PrestecsE)
                {
                    //if (PE.EnPrestec)
                    //{
                        prestecActual = new Prestec();
                        ConvertirEntityPrestecAPrestecContractData(PE, prestecActual);
                        elsPrestecs.Add(prestecActual);
                    //}
                }
            }
            catch
            {
                return null;
            }
            return elsPrestecs;
        }



        /// <summary>
        /// Convertim un usuari definit per el servei a UsuariEntity
        /// </summary>
        /// <parm name="llibreEntity">a convertir</param>
        /// <parm name="llibre">convertit</param>
        private void ConvertirEntityLlibreALlibreContractData(LlibreEntity llibreEntity, Llibre llibre)
        {
            llibre.LlibreId = llibreEntity.LlibreId;
            llibre.ISBN = llibreEntity.ISBN;
            llibre.Titol = llibreEntity.Titol;
            llibre.Descripcio = llibreEntity.Descripcio;
            llibre.CategoriaId = llibreEntity.CategoriaId;
            llibre.TipusLlibreId = llibreEntity.TipusLlibreId;
            llibre.AutorId = llibreEntity.AutorId;
            llibre.EditorialId = llibreEntity.EditorialId;
            llibre.Autor = llibreEntity.Autor;
            llibre.TipusLlibre = llibreEntity.TipusLlibre;
            llibre.Categoria = llibreEntity.Categoria;
            llibre.Editorial = llibreEntity.Editorial;
            llibre.EnPrestec = llibreEntity.EnPrestec;
        }

        /// <summary>
        /// Convertim un Prestec definit per el servei a PrestecEntity
        /// </summary>
        /// <parm name="prestecEntity">a convertir</param>
        /// <parm name="prestec">convertit</param>
        private void ConvertirEntityPrestecAPrestecContractData(PrestecEntity prestecEntity, Prestec prestec)
        {
            prestec.PrestecId = prestecEntity.PrestecId;
            prestec.LlibreId = prestecEntity.LlibreId;
            prestec.UsuariId = prestecEntity.UsuariId;
            prestec.DataPrestec = prestecEntity.DataPrestec;
            prestec.DataEstimada = prestecEntity.DataEstimada;
            prestec.Nota = prestecEntity.Nota;
            prestec.EnPrestec = (bool)prestecEntity.EnPrestec;
            prestec.TitolLlibre = prestecEntity.TitolLlibre;
        }

        /// <summary>
        /// Convertim un UsuariEntity a un Usuari definit pel servei 
        /// </summary>
        /// <parm name="usuari">a convertir</param>
        /// <parm name="usuariEntity">convertit</param>
        private void ConvertirEntityUsuariToUsuariContractData(UsuariEntity usuariEntity, Usuari usuari)
        {
            usuari.UsuariId = usuariEntity.UsuariId;
            usuari.Nom = usuariEntity.Nom;
            usuari.Cognom = usuariEntity.Cognom;
            usuari.Cognom2 = usuariEntity.Cognom2;
            usuari.DNI = usuariEntity.DNI;
            usuari.Direccio = usuariEntity.Direccio;
            usuari.Poblacio = usuariEntity.Poblacio;
            usuari.CodiPostal = usuariEntity.CodiPostal;
            usuari.Email = usuariEntity.Email;
            usuari.Notificacions = usuariEntity.Notificacions;
            usuari.AccesWeb = usuariEntity.AccesWeb;
        }

        /// <summary>
        /// Convertim un Autor definit pel servei a AutorEntity
        /// </summary>
        /// <parm name="autorEntity">a convertir</param>
        /// <parm name="autor">convertit</param>
        private void ConvertirAutorEntityToAutorContractData(AutorEntity autorEntity, Autor autor)
        {
            autor.AutorId = autorEntity.AutorId;
            autor.NomAutor = autorEntity.NomAutor;
        }

        /// <summary>
        /// Convertim un UsuariEntity definit a Usuari definit pel servei
        /// </summary>
        /// <parm name="llibreEntity">a convertir</param>
        /// <parm name="llibre">convertit</param>
        private void ConvertirUsuariContractDataToUsuariEntity(UsuariEntity usuariEntity, Usuari usuari)
        {
            usuariEntity.UsuariId = usuari.UsuariId;
            usuariEntity.Nom = usuari.Nom;
            usuariEntity.Cognom = usuari.Cognom;
            usuariEntity.Cognom2 = usuari.Cognom2;
            usuariEntity.DNI = usuari.DNI;
            usuariEntity.Direccio = usuari.Direccio;
            usuariEntity.Poblacio = usuari.Poblacio;
            usuariEntity.CodiPostal = usuari.CodiPostal;
            usuariEntity.Email = usuari.Email;
            usuariEntity.Notificacions = usuari.Notificacions;
            usuariEntity.AccesWeb = usuari.AccesWeb;
        }

        /// <summary>
        /// Convertim una Editorial definit pel servei a EditorialEntity
        /// </summary>
        /// <parm name="editorialEntity">a convertir</param>
        /// <parm name="editorial">convertit</param>
        private void ConvertirEditorialEntityToEditorialContractData(EditorialEntity editorialEntity, Editorial editorial)
        {
            editorial.EditorialId = editorialEntity.EditorialId;
            editorial.Nom = editorialEntity.Nom;
            editorial.Adreca = editorialEntity.Adreca;
            editorial.CodiPostal = editorialEntity.CodiPostal;
            editorial.Tfn = editorialEntity.Tfn;
            editorial.Ciutat = editorialEntity.Ciutat;
        }

        /// <summary>
        /// Convertim una Categoria definida pel servei a CategoriaEntity
        /// </summary>
        /// <parm name="categoriaEntity">a convertir</param>
        /// <parm name="categoria">convertit</param>
        private void ConvertirCategoriaEntityToCategoriaContractData(CategoriaEntity categoriaEntity, Categoria categoria)
        {
            categoria.CategoriaId = categoriaEntity.CategoriaId;
            categoria.Nom = categoriaEntity.Nom;
            categoria.Descripcio = categoriaEntity.Descripcio;
        }

        /// <summary>
        /// Convertim un tipusLlibre definit pel servei a TipusllibreEntity
        /// </summary>
        /// <parm name="tipusLlibreEntity">a convertir</param>
        /// <parm name="tipusLlibre">convertit</param>
        private void ConvertirTipusLlibreEntityToTipusLlibreContractData(TipusLlibreEntity tipusLlibreEntity, TipusLlibre tipusLlibre)
        {
            tipusLlibre.TipusLlibreId = tipusLlibreEntity.TipusLlibreId;
            tipusLlibre.Nom = tipusLlibreEntity.Nom;
        }

        /// <summary>
        /// Convertim una Configuracio definida pel servei a ConfiguracioEntity
        /// </summary>
        /// <parm name="configuracioEntity">a convertir</param>
        /// <parm name="configuracio">convertit</param>
        private void ConvertirEntityConfiguracioToConfiguracioContractData(ConfiguracioEntity configuracioEntity, Configuracio configuracio)
        {
            configuracio.ConfiguracioId = configuracioEntity.ConfiguracioId;
            configuracio.DiesPrestec = configuracioEntity.DiesPrestec;
            configuracio.NomBiblioteca = configuracioEntity.NomBiblioteca;
            configuracio.EmailContacte = configuracioEntity.EmailContacte;
            configuracio.Emailnotificacions = configuracioEntity.Emailnotificacions;
            configuracio.ServidorSMTP = configuracioEntity.ServidorSMTP;
            configuracio.PasswordSMTP = configuracioEntity.PasswordSMTP;
            configuracio.NotificacionsActives = configuracioEntity.NotificacionsActives;
            configuracio.UsuariSMTP = configuracioEntity.UsuariSMTP;
            configuracio.PortSMTP = configuracioEntity.PortSMTP;
        }
    }
}
