﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;

namespace Biblioteca.BibliotecaServices
{
    [ServiceContract]
    public interface IBibliotecaService
    {
        [OperationContract]
        IList<Llibre> GetLlibres(string titolSearch, string descripcioSearch, string AutorId, string EditorialId, string CategoriaId, string TipusLlibreId);

        [OperationContract]
        IList<Prestec> GetPrestecs(int usuariId);

        [OperationContract]
        Llibre GetLlibre(int llibreId);

        [OperationContract]
        IList<Autor> GetAutors();

        [OperationContract]
        Usuari GetUsuari(int usuariId);

        [OperationContract]
        IList<Editorial> GetEditorials();

        [OperationContract]
        IList<Categoria> GetCategories();

        [OperationContract]
        IList<TipusLlibre> GetTipusLlibre();

        [OperationContract]
        Configuracio GetConfiguracio();

        [OperationContract]
        int Login(string usuari, string password);

        [OperationContract]
        [FaultContract(typeof(ErrorUsuariDataIncorrecta))]
        String UpdateUsuari(Usuari usuariIn);

       /* [OperationContract]
        Usuari GetUsuariPhone(string usuari, string password);*/

        [OperationContract]
        Usuari GetUsuariPhonev2(string usuari, string password);
        /*
        [OperationContract]
        String UpdateUsuariPhone(string nomUsuari, string password, Usuari usuari);
        */
        [OperationContract]
        IList<Prestec> GetPrestecsPhone(string usuari, string password);

    }

    [DataContract]
    public class Usuari
    {
        [DataMember]
        public int UsuariId { get; set; }
        [DataMember]
        public string Nom { get; set; }
        [DataMember]
        public string Cognom { get; set; }
        [DataMember]
        public string Cognom2 { get; set; }
        [DataMember]
        public string DNI { get; set; }
        [DataMember]
        public string Direccio { get; set; }
        [DataMember]
        public string Poblacio { get; set; }
        [DataMember]
        public string CodiPostal { get; set; }
        [DataMember]
        public string Email { get; set; }
        [DataMember]
        public bool Notificacions { get; set; }
        [DataMember]
        public bool AccesWeb { get; set; }
    }

    [DataContract]
    public class TipusUsuari
    {
        [DataMember]
        public int TipusUsuariId { get; set; }
        [DataMember]
        public string NomTipusUsuari { get; set; }
        [DataMember]
        public string DescripcioTipusUsuari { get; set; }
    }

    [DataContract]
    [Serializable]
    public class Llibre
    {
        [DataMember]
        public int LlibreId { get; set; }
        [DataMember]
        public string ISBN { get; set; }
        [DataMember]
        public string Titol { get; set; }
        [DataMember]
        public string Descripcio { get; set; }
        [DataMember]
        public int CategoriaId { get; set; }
        [DataMember]
        public string Categoria { get; set; }
        [DataMember]
        public int EditorialId { get; set; }
        [DataMember]
        public string Editorial { get; set; }
        [DataMember]
        public int TipusLlibreId { get; set; }
        [DataMember]
        public string TipusLlibre { get; set; }
        [DataMember]
        public int AutorId { get; set; }
        [DataMember]
        public string Autor { get; set; }
        [DataMember]
        public bool EnPrestec { get; set; }
    }

    [DataContract]
    [Serializable]
    public class Prestec
    {
        [DataMember]
        public int PrestecId { get; set; }
        [DataMember]
        public int LlibreId { get; set; }
        [DataMember]
        public int UsuariId { get; set; }
        [DataMember]
        public DateTime DataPrestec { get; set; }
        [DataMember]
        public DateTime DataEstimada { get; set; }
        [DataMember]
        public string Nota { get; set; }
        [DataMember]
        public bool EnPrestec { get; set; }
        [DataMember]
        public string TitolLlibre { get; set; }
    }

    [DataContract]
    [Serializable]
    public class Autor
    {
        [DataMember]
        public int AutorId { get; set; }
        [DataMember]
        public string NomAutor { get; set; }
    }

    [DataContract]
    [Serializable]
    public class Editorial
    {
        [DataMember]
        public int EditorialId { get; set; }
        [DataMember]
        public string Nom { get; set; }
        [DataMember]
        public string Adreca { get; set; }
        [DataMember]
        public string CodiPostal { get; set; }
        [DataMember]
        public string Tfn { get; set; }
        [DataMember]
        public string Ciutat { get; set; }
    }

    [DataContract]
    [Serializable]
    public class Categoria
    {
        [DataMember]
        public int CategoriaId { get; set; }
        [DataMember]
        public string Nom { get; set; }
        [DataMember]
        public string Descripcio { get; set; }
    }

    [DataContract]
    [Serializable]
    public class TipusLlibre
    {
        [DataMember]
        public int TipusLlibreId { get; set; }
        [DataMember]
        public string Nom { get; set; }
    }

    [DataContract]
    [Serializable]
    public class Configuracio
    {
        [DataMember]
        public int ConfiguracioId { get; set; }
        [DataMember]
        public int DiesPrestec { get; set; }
        [DataMember]
        public string NomBiblioteca { get; set; }
        [DataMember]
        public string EmailContacte { get; set; }
        [DataMember]
        public string Emailnotificacions { get; set; }
        [DataMember]
        public string ServidorSMTP { get; set; }
        [DataMember]
        public string PasswordSMTP { get; set; }
        [DataMember]
        public bool NotificacionsActives { get; set; }
        [DataMember]
        public string UsuariSMTP { get; set; }
        [DataMember]
        public int PortSMTP { get; set; }
    }

    [DataContract]
    public class ErrorUsuariDataIncorrecta
    {
        public ErrorUsuariDataIncorrecta(string msg)
        {
            FaultMessage = msg;
        }
        [DataMember]
        public string FaultMessage;
    }
}
