﻿/// <summary>
/// Aquesta és la classe responsable de controlar el tipus d'accés i les pàgines (views) que se'n deriven
/// </summary>
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using WebAdmin.Models;

namespace WebAdmin.Controllers
{
    public class AccesTipuController : Controller
    {
        private BibliotecaEntities1 db = new BibliotecaEntities1();

        /// <summary>
        /// GET: /AccesTipu/
        /// Mostrem un llistat de tipus d'usuari
        /// </summary>
        public ActionResult Index()
        {
            var accestipus = db.AccesTipus.Include("TipusLlibre").Include("TipusUsuari");
            return View(accestipus.ToList());
        }

        /// <summary>
        /// GET: /AccesTipu/Details/5
        /// Mostrem el tipus d'accés
        /// </summary>
        /// <parm name="id">l'identificador del tipus d'accés a mostrar</param>
        public ActionResult Details(int id = 0)
        {
            AccesTipu accestipu = db.AccesTipus.Single(a => a.AccesTipuId == id);
            if (accestipu == null)
            {
                return HttpNotFound();
            }
            return View(accestipu);
        }


        /// <summary>
        /// GET: /AccesTipu/Create
        /// Mostrem la vista que permet introduir les dades d'un nou tipus d'accés
        /// </summary>
        public ActionResult Create()
        {
            ViewBag.TipusLlibreId = new SelectList(db.TipusLlibres, "TipusLlibreId", "Nom");
            ViewBag.TipusUsuariId = new SelectList(db.TipusUsuaris, "TipusUsuariId", "NomTipusUsuari");
            return View();
        }


        /// <summary>
        /// POST: /AccesTipu/Create
        /// Recollim les dades del tipus d'accés introduit i les guardem
        /// </summary>
        /// <parm name="accestipu">dades del tipus d'accés des del formulari de creació via el model AccesTipu</param>
        [HttpPost]
        public ActionResult Create(AccesTipu accestipu)
        {
            if (ModelState.IsValid)
            {
                // Busquem si l'accés introduit ja existeix
                var acces = from p in db.AccesTipus
                              select p;

                acces = acces.Where(p => p.TipusUsuariId == accestipu.TipusUsuariId);
                acces = acces.Where(p => p.TipusLlibreId == accestipu.TipusLlibreId);
                acces = acces.Include("TipusUsuari");
                acces = acces.Include("TipusLlibre");

                if (acces.Count() != 0)
                {
                    // L'accés existeix i per tant reenviem a la pàgina d'error
                    List<String> ErrorList = new List<String>();
                    ErrorList.Add("Ja existeix un accés definit per aquest Tipus d'usuari / Tipus de llibre ");
                    foreach (var p in acces.ToList())
                    {
                        ErrorList.Add(p.TipusLlibre.Nom + " / " + p.TipusUsuari.NomTipusUsuari + " " + p.Access.ToString());
                    }
                    ViewBag.errorList = ErrorList;
                    ViewBag.errorTitle = "Error Creant permisos d'accés";
                    ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                    return View("../Error/Error");
                }
                // Afegim el tipus d'accés
                db.AccesTipus.AddObject(accestipu);
                db.SaveChanges();
                return RedirectToAction("Index");
            }

            ViewBag.TipusLlibreId = new SelectList(db.TipusLlibres, "TipusLlibreId", "Nom", accestipu.TipusLlibreId);
            ViewBag.TipusUsuariId = new SelectList(db.TipusUsuaris, "TipusUsuariId", "NomTipusUsuari", accestipu.TipusUsuariId);
            return View(accestipu);
        }


        /// <summary>
        /// GET: /AccesTipu/Edit/5
        /// Editem el tipus d'accés
        /// </summary>
        /// <parm name="id">L'identificador del tipus d'accés</param>
        public ActionResult Edit(int id = 0)
        {
            AccesTipu accestipu = db.AccesTipus.Single(a => a.AccesTipuId == id);
            if (accestipu == null)
            {
                return HttpNotFound();
            }
            ViewBag.TipusLlibreId = new SelectList(db.TipusLlibres, "TipusLlibreId", "Nom", accestipu.TipusLlibreId);
            ViewBag.TipusUsuariId = new SelectList(db.TipusUsuaris, "TipusUsuariId", "NomTipusUsuari", accestipu.TipusUsuariId);
            return View(accestipu);
        }

        /// <summary>
        /// POST: /AccesTipu/Edit/5
        /// Recollim les modificacions dell tipus d'accés introduit i les actualitzem
        /// </summary>
        /// <parm name="accestipu">dades del tipus d'accés des del formulari de edició via el model AccesTipu</param>
        [HttpPost]
        public ActionResult Edit(AccesTipu accestipu)
        {
            if (ModelState.IsValid)
            {
                db.AccesTipus.Attach(accestipu);
                db.ObjectStateManager.ChangeObjectState(accestipu, EntityState.Modified);
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            ViewBag.TipusLlibreId = new SelectList(db.TipusLlibres, "TipusLlibreId", "Nom", accestipu.TipusLlibreId);
            ViewBag.TipusUsuariId = new SelectList(db.TipusUsuaris, "TipusUsuariId", "NomTipusUsuari", accestipu.TipusUsuariId);
            return View(accestipu);
        }


        /// <summary>
        /// GET: /AccesTipu/Delete/5
        /// Eliminem el tipus d'accés
        /// </summary>
        /// <parm name="id">Identificador del tipus d'accés</param>
        public ActionResult Delete(int id = 0)
        {
            AccesTipu accestipu = db.AccesTipus.Single(a => a.AccesTipuId == id);
            if (accestipu == null)
            {
                return HttpNotFound();
            }
            return View(accestipu);
        }

        /// <summary>
        /// POST: /AccesTipu/Delete/5
        /// Eliminem el tipus d'accés
        /// </summary>
        /// <parm name="id">Identificador del tipus d'accés a eliminar</param>
        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            AccesTipu accestipu = db.AccesTipus.Single(a => a.AccesTipuId == id);
            db.AccesTipus.DeleteObject(accestipu);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}