﻿/// <summary>
/// Aquesta és la classe responsable de controlar l'autor i les pàgines (vistes) que se'n deriven
/// </summary>
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using WebAdmin.Models;

namespace WebAdmin.Controllers
{
    public class AutorController : Controller
    {
        private BibliotecaEntities1 db = new BibliotecaEntities1();


        /// <summary>
        /// GET: /Autor/
        /// Mostrem un llistat d'autors
        /// </summary>
        public ActionResult Index()
        {
            return View(db.Autors.ToList());
        }


        /// <summary>
        /// GET: /Autor/Details/5
        /// Mostrem l'autor
        /// </summary>
        /// <parm name="id">l'identificador de l'autor a mostrar</param>
        public ActionResult Details(int id = 0)
        {
            Autor autor = db.Autors.Single(a => a.AutorId == id);
            if (autor == null)
            {
                return HttpNotFound();
            }
            return View(autor);
        }


        /// <summary>
        /// GET: /Autor/Create
        /// Mostrem la vista que permet introduir les dades d'autor
        /// </summary>
        public ActionResult Create()
        {
            return View();
        }


        /// <summary>
        /// POST: /Autor/Create
        /// Recollim les dades de l'autor introduit i les guardem
        /// </summary>
        /// <parm name="autor">dades de l'autor des del formulari de creació via el model Autor</param>
        [HttpPost]
        public ActionResult Create(Autor autor)
        {
            if (ModelState.IsValid)
            {
                db.Autors.AddObject(autor);
                db.SaveChanges();
                return RedirectToAction("Index");
            }

            return View(autor);
        }


        /// <summary>
        /// GET: /Autor/Edit/5
        /// Editem l'autor
        /// </summary>
        /// <parm name="id">l'identificador de l'autor</param>
        public ActionResult Edit(int id = 0)
        {
            Autor autor = db.Autors.Single(a => a.AutorId == id);
            if (autor == null)
            {
                return HttpNotFound();
            }
            return View(autor);
        }

        /// <summary>
        /// POST: /Autor/Edit/
        /// Recollim les modificacions de l'autor introduit i les actualitzem
        /// </summary>
        /// <parm name="autor">dades de l'autor des del formulari d'edició via el model Autor</param>
        [HttpPost]
        public ActionResult Edit(Autor autor)
        {
            if (ModelState.IsValid)
            {
                db.Autors.Attach(autor);
                db.ObjectStateManager.ChangeObjectState(autor, EntityState.Modified);
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            return View(autor);
        }


        /// <summary>
        /// GET: /Autor/Delete/5
        /// Eliminem l'autor, mostrem els detalls i mostrem el formulari de verificació
        /// </summary>
        /// <parm name="id">identificador de l'autor</param>
        public ActionResult Delete(int id = 0)
        {
            //Verifiquem que no hi hagin llibres de l'autor a eliminar
            var llibres = from p in db.Llibres
                           select p;

            llibres = llibres.Where(p => p.AutorId == id);
            if (llibres.Count() != 0)
            {
                List<String> ErrorList = new List<String>();
                ErrorList.Add("Existeixen Llibres de l'autor a eliminar, siusplau elimina els llibres abans de procedir a l'eliminació de l'autor");
                foreach (var p in llibres.ToList())
                {
                    ErrorList.Add("   - LlibreId: " + p.LlibreId.ToString() + " " + p.Titol + " " + p.ISBN.ToString());
                }
                ViewBag.errorList = ErrorList;
                ViewBag.errorTitle = "Error Eliminant l'autor";
                ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                return View("../Error/Error");
            }
            Autor autor = db.Autors.Single(a => a.AutorId == id);
            if (autor == null)
            {
                return HttpNotFound();
            }
            return View(autor);
        }


        /// <summary>
        /// POST: /Autor/Delete/5
        /// Eliminem l'autor
        /// </summary>
        /// <parm name="id">identificador de l'autor a eliminar</param>
        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            Autor autor = db.Autors.Single(a => a.AutorId == id);
            db.Autors.DeleteObject(autor);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}