﻿/// <summary>
/// Aquesta és la classe responsable de controlar les categories i les pàgines (vistes) que se'n deriven
/// </summary>
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using WebAdmin.Models;

namespace WebAdmin.Controllers
{
    public class CategoriaController : Controller
    {
        private BibliotecaEntities1 db = new BibliotecaEntities1();


        /// <summary>
        /// GET: /Categoria/
        /// Mostrem una llista de categories
        /// </summary>
        public ActionResult Index()
        {
            return View(db.Categorias.ToList());
        }

        /// <summary>
        /// GET: /Categoria/Details/5
        /// Mostrem la categoria
        /// </summary>
        /// <parm name="id">l'identificador de la categoria</param>
        public ActionResult Details(int id = 0)
        {
            Categoria categoria = db.Categorias.Single(c => c.CategoriaId == id);
            if (categoria == null)
            {
                return HttpNotFound();
            }
            return View(categoria);
        }


        /// <summary>
        /// GET: /Categoria/Create
        /// Mostrem la vista que permet introduir la categoria
        /// </summary>
        public ActionResult Create()
        {
            return View();
        }


        /// <summary>
        /// POST: /Categoria/Create
        /// Recollim les dades de la categoria introduida i les guardem
        /// </summary>
        /// <parm name="categoria">dades de la categoria des del formulari de creació via el model Categoria</param>
        [HttpPost]
        public ActionResult Create(Categoria categoria)
        {
            if (ModelState.IsValid)
            {
                db.Categorias.AddObject(categoria);
                db.SaveChanges();
                return RedirectToAction("Index");
            }

            return View(categoria);
        }


        /// <summary>
        /// GET: /Categoria/Edit/5
        /// Editem la categoria
        /// </summary>
        /// <parm name="id">l'identificador de la categoria</param>
        public ActionResult Edit(int id = 0)
        {
            Categoria categoria = db.Categorias.Single(c => c.CategoriaId == id);
            if (categoria == null)
            {
                return HttpNotFound();
            }
            return View(categoria);
        }


        /// <summary>
        /// POST: /Categoria/Edit/5
        /// Recollim les modificacions de categoria introduida i les actualitzem
        /// </summary>
        /// <parm name="categoria">dades de la categoria des del formulari d'edició via el model Categoria</param>
        [HttpPost]
        public ActionResult Edit(Categoria categoria)
        {
            if (ModelState.IsValid)
            {
                db.Categorias.Attach(categoria);
                db.ObjectStateManager.ChangeObjectState(categoria, EntityState.Modified);
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            return View(categoria);
        }


        /// <summary>
        /// GET: /Categoria/Delete/5
        /// Eliminem la categoria
        /// </summary>
        /// <parm name="id">identificador de categoria</param>
        public ActionResult Delete(int id = 0)
        {
            var llibres = from p in db.Llibres
                          select p;

            llibres = llibres.Where(p => p.CategoriaId == id);
            if (llibres.Count() != 0)
            {
                List<String> ErrorList = new List<String>();
                ErrorList.Add("Existeixen Llibres de la categoria a eliminar, siusplau elimina/canvia la categoria dels llibres abans de procedir a l'eliminació de la categoria");
                foreach (var p in llibres.ToList())
                {
                    ErrorList.Add("   - LlibreId: " + p.LlibreId.ToString() + " " + p.Titol + " " + p.ISBN.ToString());
                }
                ViewBag.errorList = ErrorList;
                ViewBag.errorTitle = "Error Eliminant la Categoria";
                ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                return View("../Error/Error");
            }
            Categoria categoria = db.Categorias.Single(c => c.CategoriaId == id);
            if (categoria == null)
            {
                return HttpNotFound();
            }
            return View(categoria);
        }


        /// <summary>
        /// POST: /Categoria/Delete/5
        /// Eliminem la categoria
        /// </summary>
        /// <parm name="id">identificador de la categoria a eliminar</param>
        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            Categoria categoria = db.Categorias.Single(c => c.CategoriaId == id);
            db.Categorias.DeleteObject(categoria);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}