﻿/// <summary>
/// Aquesta és la classe responsable de controlar les configuracions del sistema i les pàgines (vistes) que se'n deriven
/// </summary>
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using WebAdmin.Models;

namespace WebAdmin.Controllers
{
    public class ConfiguracioController : Controller
    {
        private BibliotecaEntities1 db = new BibliotecaEntities1();


        /// <summary>
        /// GET: /Configuracio/
        /// Mostrem un llistat de configuracions
        /// </summary>
        public ActionResult Index()
        {
            return View(db.Configuracios.ToList());
        }


        /// <summary>
        /// GET: /Configuracio/Details/5
        /// Mostrem la configuració
        /// </summary>
        /// <parm name="id">l'identificador de la configuració</param>
        public ActionResult Details(int id = 1)
        {
            Configuracio configuracio = db.Configuracios.Single(c => c.ConfiguracioId == id);
            if (configuracio == null)
            {
                return HttpNotFound();
            }
            return View(configuracio);
        }


        /// <summary>
        /// GET: /Configuracio/Create
        /// Mostrem la vista que permet introduir les dades de configuració
        /// </summary>
        public ActionResult Create()
        {
            return View();
        }


        /// <summary>
        // POST: /Configuracio/Create
        /// Recollim les dades de la configuració i les guardem
        /// </summary>
        /// <parm name="configuracio">dades la configuracio des del formulari de creació via el model Configuracio</param>
        [HttpPost]
        public ActionResult Create(Configuracio configuracio)
        {
            if (ModelState.IsValid)
            {
                db.Configuracios.AddObject(configuracio);
                db.SaveChanges();
                return RedirectToAction("Index");
            }

            return View(configuracio);
        }


        /// <summary>
        /// GET: /Configuracio/Edit/5
        /// Editem la configuració
        /// </summary>
        /// <parm name="id">l'identificador de la configuració</param>
        public ActionResult Edit(int id = 0)
        {
            Configuracio configuracio = db.Configuracios.Single(c => c.ConfiguracioId == id);
            if (configuracio == null)
            {
                return HttpNotFound();
            }
            return View(configuracio);
        }


        /// <summary>
        /// POST: /Configuracio/Edit/5
        /// Recollim les modificacions de la configuració introduida i les actualitzem
        /// </summary>
        /// <parm name="configuracio">dades de la configuracio des del formulari d'edició via el model Configuracio</param>
        [HttpPost]
        public ActionResult Edit(Configuracio configuracio)
        {
            if (ModelState.IsValid)
            {
                db.Configuracios.Attach(configuracio);
                db.ObjectStateManager.ChangeObjectState(configuracio, EntityState.Modified);
                db.SaveChanges();
                return RedirectToAction("../Home/Index");
            }
            return View(configuracio);
        }


/*        /// <summary>
        /// GET: /Configuracio/Delete/5
        /// Eliminem la , mostrem els detalls i mostrem el formulari de verificació
        /// </summary>
        /// <parm name="id">identificador de l'autor</param>
        public ActionResult Delete(int id = 0)
        {
            Configuracio configuracio = db.Configuracios.Single(c => c.ConfiguracioId == id);
            if (configuracio == null)
            {
                return HttpNotFound();
            }
            return View(configuracio);
        }

        //
        // POST: /Configuracio/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            Configuracio configuracio = db.Configuracios.Single(c => c.ConfiguracioId == id);
            db.Configuracios.DeleteObject(configuracio);
            db.SaveChanges();
            return RedirectToAction("Index");
        }*/

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}