﻿/// <summary>
/// Aquesta és la classe responsable de controlar les Editorials i les pàgines (vistes) que se'n deriven
/// </summary>
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using WebAdmin.Models;

namespace WebAdmin.Controllers
{
    public class EditorialController : Controller
    {
        private BibliotecaEntities1 db = new BibliotecaEntities1();

        //
        // GET: /Editorial/

        public ActionResult Index()
        {
            return View(db.Editorials.ToList());
        }

        //
        // GET: /Editorial/Details/5

        public ActionResult Details(int id = 0)
        {
            Editorial editorial = db.Editorials.Single(e => e.EditorialId == id);
            if (editorial == null)
            {
                return HttpNotFound();
            }
            return View(editorial);
        }

        //
        // GET: /Editorial/Create

        public ActionResult Create()
        {
            return View();
        }

        //
        // POST: /Editorial/Create

        [HttpPost]
        public ActionResult Create(Editorial editorial)
        {
            if (ModelState.IsValid)
            {
                db.Editorials.AddObject(editorial);
                db.SaveChanges();
                return RedirectToAction("Index");
            }

            return View(editorial);
        }

        //
        // GET: /Editorial/Edit/5

        public ActionResult Edit(int id = 0)
        {
            Editorial editorial = db.Editorials.Single(e => e.EditorialId == id);
            if (editorial == null)
            {
                return HttpNotFound();
            }
            return View(editorial);
        }

        //
        // POST: /Editorial/Edit/5

        [HttpPost]
        public ActionResult Edit(Editorial editorial)
        {
            if (ModelState.IsValid)
            {
                db.Editorials.Attach(editorial);
                db.ObjectStateManager.ChangeObjectState(editorial, EntityState.Modified);
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            return View(editorial);
        }

        //
        // GET: /Editorial/Delete/5

        public ActionResult Delete(int id = 0)
        {
            var llibres = from p in db.Llibres
                          select p;

            llibres = llibres.Where(p => p.EditorialId == id);
            if (llibres.Count() != 0)
            {
                List<String> ErrorList = new List<String>();
                ErrorList.Add("Existeixen Llibres de l'editorial a eliminar, siusplau elimina els llibres o canvia la editorial abans de procedir a l'eliminació de la mateixa");
                foreach (var p in llibres.ToList())
                {
                    ErrorList.Add("   - LlibreId: " + p.LlibreId.ToString() + " " + p.Titol + " " + p.ISBN.ToString());
                }
                ViewBag.errorList = ErrorList;
                ViewBag.errorTitle = "Error Eliminant l'editorial";
                ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                return View("../Error/Error");
            }
            Editorial editorial = db.Editorials.Single(e => e.EditorialId == id);
            if (editorial == null)
            {
                return HttpNotFound();
            }
            return View(editorial);
        }

        //
        // POST: /Editorial/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            Editorial editorial = db.Editorials.Single(e => e.EditorialId == id);
            db.Editorials.DeleteObject(editorial);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}