﻿/// <summary>
/// Aquesta és la classe responsable de controlar els llibres les pàgines (vistes) que se'n deriven
/// </summary>
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using WebAdmin.Models;
using PagedList;
using System.Text;

namespace WebAdmin.Controllers
{
    public class LlibreController : Controller
    {
        private BibliotecaEntities1 db = new BibliotecaEntities1();


        /// <summary>
        /// GET: /Llibre/
        /// Mostrem un llistat de llibres
        /// </summary>
        /// <parm name="searchStringISBN">Filtre per ISBN</param>
        /// <parm name="searchStringTitol">Filtre per Titol</param>
        /// <parm name="AutorId">Filtre per Autor</param>
        /// <parm name="page">Número de pàgina de la paginació</param>
        public ActionResult Index(string searchStringISBN, string searchStringTitol, string AutorId, int? page)
        {
            if (Request.HttpMethod != "GET")
            {
                page = 1;
            }
            
            // Recollim els filtres de cerca i els coloquem a la ViewBag per tal de que quan obrim la vista amb els resultats també hi apareixin els criteris de búsqueda
            ViewBag.CurrentFilterISBN = searchStringISBN;
            ViewBag.CurrentFilterTitol = searchStringTitol;
            ViewBag.CurrentFilterAutor = AutorId;

            string currentUserStr = ViewBag.currentUser;
            
            ViewBag.AutorId = new SelectList(db.Autors, "AutorId", "NomAutor");

            var llibres = from l in db.Llibres
                           select l;
            llibres = llibres.OrderBy(u => u.Autor.NomAutor);

            // Filtrem per ISBN
            if (!String.IsNullOrEmpty(searchStringISBN))
            {
                llibres = llibres.Where(l => l.ISBN.ToUpper().Contains(searchStringISBN.ToUpper()));
            }
            // Filtrem per títol
            if (!String.IsNullOrEmpty(searchStringTitol))
            {
                llibres = llibres.Where(l => l.Titol.ToUpper().Contains(searchStringTitol.ToUpper()));
            }
            int currentAutor;
            //Filtrem per autor
            if (!String.IsNullOrEmpty(AutorId))
            {
                currentAutor = Convert.ToInt32(AutorId);
                llibres = llibres.Where(l => l.AutorId == currentAutor);
            }
            int pageSize = 10;
            int pageNumber = (page ?? 1);
            int total = llibres.Count();
            if (total == 0)
                ViewBag.IsEmpty = true;
            else
                ViewBag.IsEmpty = false;
            llibres = llibres.OrderBy(l => l.Autor.NomAutor);
            return View(llibres.ToPagedList(pageNumber, pageSize));
        }


        /// <summary>
        /// GET: /Llibre/Details/5
        /// Mostrem el llibre
        /// </summary>
        /// <parm name="id">l'identificador del llibre a mostrar</param>
        public ActionResult Details(int id = 0)
        {
            Llibre llibre = db.Llibres.Single(l => l.LlibreId == id);
            if (llibre == null)
            {
                return HttpNotFound();
            }
            return View(llibre);
        }


        /// <summary>
        /// GET: /Llibre/Create
        /// Mostrem la vista que permet introduir les dades del llibre
        /// </summary>
        public ActionResult Create()
        {
            ViewBag.AutorId = new SelectList(db.Autors, "AutorId", "NomAutor");
            ViewBag.CategoriaId = new SelectList(db.Categorias, "CategoriaId", "Nom");
            ViewBag.EditorialId = new SelectList(db.Editorials, "EditorialId", "Nom");
            ViewBag.TipusLlibreId = new SelectList(db.TipusLlibres, "TipusLlibreId", "Nom");
            return View();
        }


        /// <summary>
        /// POST: /Llibre/Create
        /// Recollim les dades del llibre introduit i les guardem
        /// </summary>
        /// <parm name="llibre">dades del llibre des del formulari de creació via el model Llibre</param>
        [HttpPost]
        public ActionResult Create(Llibre llibre)
        {
            if (ModelState.IsValid)
            {
                db.Llibres.AddObject(llibre);
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            ViewBag.AutorId = new SelectList(db.Autors, "AutorId", "NomAutor", llibre.AutorId);
            ViewBag.CategoriaId = new SelectList(db.Categorias, "CategoriaId", "Nom", llibre.CategoriaId);
            ViewBag.EditorialId = new SelectList(db.Editorials, "EditorialId", "Nom", llibre.EditorialId);
            ViewBag.TipusLlibreId = new SelectList(db.TipusLlibres, "TipusLlibreId", "Nom", llibre.TipusLlibreId);
            return View(llibre);
        }


        /// <summary>
        /// GET: /Autor/Edit/5
        /// Editem el llibre
        /// </summary>
        /// <parm name="id">l'identificador de llibre</param>
        public ActionResult Edit(int id = 0)
        {
            Llibre llibre = db.Llibres.Single(l => l.LlibreId == id);
            if (llibre == null)
            {
                return HttpNotFound();
            }
            ViewBag.AutorId = new SelectList(db.Autors, "AutorId", "NomAutor", llibre.AutorId);
            ViewBag.CategoriaId = new SelectList(db.Categorias, "CategoriaId", "Nom", llibre.CategoriaId);
            ViewBag.EditorialId = new SelectList(db.Editorials, "EditorialId", "Nom", llibre.EditorialId);
            ViewBag.TipusLlibreId = new SelectList(db.TipusLlibres, "TipusLlibreId", "Nom", llibre.TipusLlibreId);
            return View(llibre);
        }


        /// <summary>
        /// POST:  /Llibre/Edit/5
        /// Recollim les modificacions del llibre introduit i les actualitzem
        /// </summary>
        /// <parm name="llibre">dades del llibre des del formulari d'edició via el model Llibre</param>
        [HttpPost]
        public ActionResult Edit(Llibre llibre, HttpPostedFileBase file)
        {
            if (ModelState.IsValid)
            {
                db.Llibres.Attach(llibre);
                db.ObjectStateManager.ChangeObjectState(llibre, EntityState.Modified);
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            ViewBag.AutorId = new SelectList(db.Autors, "AutorId", "NomAutor", llibre.AutorId);
            ViewBag.CategoriaId = new SelectList(db.Categorias, "CategoriaId", "Nom", llibre.CategoriaId);
            ViewBag.EditorialId = new SelectList(db.Editorials, "EditorialId", "Nom", llibre.EditorialId);
            ViewBag.TipusLlibreId = new SelectList(db.TipusLlibres, "TipusLlibreId", "Nom", llibre.TipusLlibreId);
            return View(llibre);
        }


        /// <summary>
        /// GET: /Llibre/Delete/5
        /// Eliminem el llibre, mostrem el formulari de verificació
        /// </summary>
        /// <parm name="id">identificador de llibre</param>
        public ActionResult Delete(int id = 0)
        {
            var prestecs = from p in db.Prestecs
                           select p;

            prestecs = prestecs.Where(p => p.LlibreId == id);
            prestecs = prestecs.Include("Usuari");
            // Ens assegurem que no hi han prèstecs actius associats al llibre
            if (prestecs.Count() != 0)
            {
                List<String> ErrorList = new List<String>();
                ErrorList.Add("Existeixen prèstecs associats al llibre, siusplau elimina els prèstecs abans de procedir amb l'eliminació del llibre");
                foreach (var p in prestecs.ToList())
                {
                    ErrorList.Add("   - PrestedId: " + p.PrestecId.ToString() + " - Usuari: " + p.Usuari.Nom + " " + p.Usuari.Cognom + " " + p.Usuari.Cognom2);
                }
                ViewBag.errorList = ErrorList;
                ViewBag.errorTitle = "Error Eliminant el llibre";
                ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                return View("../Error/Error");
            }
            Llibre llibre = db.Llibres.Single(l => l.LlibreId == id);
            if (llibre == null)
            {
                return HttpNotFound();
            }
            return View(llibre);
        }

        /// <summary>
        /// POST: /Llibre/Delete/5
        /// Eliminem el llibre
        /// </summary>
        /// <parm name="id">identificador del llibre a eliminar</param>
        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            Llibre llibre = db.Llibres.Single(l => l.LlibreId == id);
            db.Llibres.DeleteObject(llibre);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}