﻿/// <summary>
/// Aquesta és la classe responsable de controlar els préstecs i les pàgines (vistes) que se'n deriven
/// </summary>
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using WebAdmin.Models;

namespace WebAdmin.Controllers
{
    public class PrestecController : Controller
    {
        private BibliotecaEntities1 db = new BibliotecaEntities1();


        /// <summary>
        /// GET: /Prestec/
        /// Mostrem un llistat de préstecs en curs
        /// </summary>
        public ActionResult Index()
        {
            var prestecs = from a in db.Prestecs
                           select a;
            prestecs = prestecs.Where(p => p.EnPrestec == true);
            prestecs = prestecs.OrderByDescending(p => p.dataPrestec);
            prestecs = prestecs.Include("Llibre");
            prestecs = prestecs.Include("Usuari");
            return View(prestecs.ToList());
        }


        /// <summary>
        /// GET: /Prestec/IndexHistoric
        /// Mostrem un llistat de préstecs historics
        /// </summary>
        public ActionResult IndexHistoric()
        {
            ////var prestecs = db.Prestecs.Include("Llibre").Include("Usuari");

            var prestecs = from a in db.Prestecs
                        select a;
            prestecs = prestecs.Where(p => p.EnPrestec == false);
            prestecs = prestecs.OrderByDescending(p => p.dataPrestec);
            prestecs = prestecs.Include("Llibre");
            prestecs = prestecs.Include("Usuari");
            return View(prestecs.ToList());
        }

 
        /// <summary>
        /// GET: /Prestec/Details/5
        /// Mostrem el prèstec
        /// </summary>
        /// <parm name="id">l'identificador del préstec a mostrar</param>
        public ActionResult Details(int id = 0)
        {
            Prestec prestec = this.db.Prestecs.Single(p => p.PrestecId == id);
            if (prestec == null)
            {
                return HttpNotFound();
            }
            return View(prestec);
        }



        /// <summary>
        /// POST: /Prestec/Create
        /// Recollim les dades del préstec i realitzem el préstec
        /// </summary>
        /// <parm name="llibreId">identificador del llibre prestat</param>
        /// <parm name="usuariId">identificador de l'usuari que realitza el préstec</param>
        public ActionResult Create(int llibreId, int usuariId)
        {
            //Verifiquem que totes les dades són correctes i en cas de no ser-ho preparem les dades a enviar a la vista d'error.
            bool IsError = false;
            List<String> ErrorList = new List<String>();

            // Verifiquem que el llibre existeix
            Llibre llibre = this.db.Llibres.SingleOrDefault(l => l.LlibreId == llibreId);
            if (llibre == null)
            {
                IsError = true;
                ErrorList.Add("El llibre no existeix");

            }
            // Verifiquem que l'usuari existeix
            Usuari usuari = db.Usuaris.SingleOrDefault(u => u.UsuariId == usuariId);
            if (usuari == null)
            {
                IsError = true;
                ErrorList.Add("L'usuari no existeix");
            }

            if (IsError)
            {
                ViewBag.errorList = ErrorList;
                ViewBag.errorTitle = "Error creant Prèstec";
                ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                return View("../Error/Error");
            }
            else
            {
                // Verifiquem que el llibre no es trobi en préstec
                if (llibre.EnPrestec)
                {
                    IsError = true;
                    ErrorList.Add("El Llibre ja es troba en prèstec");
                    ViewBag.errorList = ErrorList;
                    ViewBag.errorTitle = "Error creant Prèstec";
                    ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                    return View("../Error/Error");
                }
            }

            // Realitzem el préstec
            Prestec nouPrestec = new Prestec();
            nouPrestec.UsuariId = usuariId;
            nouPrestec.LlibreId = llibreId;
            nouPrestec.dataPrestec = DateTime.Now;
            Configuracio config = this.db.Configuracios.SingleOrDefault(c => c.ConfiguracioId == 1);
            nouPrestec.dataEstimada = DateTime.Now.AddDays((int)config.DiesPrestec);
            nouPrestec.EnPrestec = true;
            if (ModelState.IsValid)
            {
                db.Prestecs.AddObject(nouPrestec);
                llibre.EnPrestec = true;
                db.SaveChanges();
                return RedirectToAction("../Usuari/Details", new { id = usuari.UsuariId });
            }
            else
            {
                IsError = true;
                ErrorList.Add("Error Desconegut");
                ViewBag.errorList = ErrorList;
                ViewBag.errorTitle = "Error Desconegut";
                ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                return View("../Error/Error");
            }
        }


        /// <summary>
        /// GET: /Prestec/Edit/5
        /// Editem el préstec
        /// </summary>
        /// <parm name="id">l'identificador del préstec</param>
        public ActionResult Edit(int id = 0)
        {
            Prestec prestec = db.Prestecs.Single(p => p.PrestecId == id);
            if (prestec == null)
            {
                return HttpNotFound();
            }
            ViewBag.LlibreId = new SelectList(db.Llibres, "LlibreId", "ISBN", prestec.LlibreId);
            ViewBag.UsuariId = new SelectList(db.Usuaris, "UsuariId", "Nom", prestec.UsuariId);
            return View(prestec);
        }


        /// <summary>
        /// POST: /Prestec/Edit/5
        /// Recollim les modificacions del préstec introduit i les actualitzem
        /// </summary>
        /// <parm name="prestec">dades del préstec des del formulari d'edició via el model Prestec</param>
        [HttpPost]
        public ActionResult Edit(Prestec prestec)
        {
            if (ModelState.IsValid)
            {
                db.Prestecs.Attach(prestec);
                db.ObjectStateManager.ChangeObjectState(prestec, EntityState.Modified);
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            ViewBag.LlibreId = new SelectList(db.Llibres, "LlibreId", "ISBN", prestec.LlibreId);
            ViewBag.UsuariId = new SelectList(db.Usuaris, "UsuariId", "Nom", prestec.UsuariId);
            return View(prestec);
        }


        /// <summary>
        /// GET: /Prestec/Delete/5
        /// Eliminem el préstec, en mostrem els detalls i mostrem el formulari de verificació
        /// </summary>
        /// <parm name="id">identificador del préstec</param>
        public ActionResult Delete(int id = 0)
        {
            Prestec prestec = db.Prestecs.Single(p => p.PrestecId == id);
            if (prestec == null)
            {
                return HttpNotFound();
            }
            return View(prestec);
        }


        /// <summary>
        /// POST: /Prestec/Delete/5
        /// Eliminem l'autor
        /// </summary>
        /// <parm name="id">identificador del préstec a eliminar</param>
        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            Prestec prestec = db.Prestecs.Single(p => p.PrestecId == id);
            // Si el préstec és actiu (no és un históric) alliberem el llibre per tal de que es pugui prestar de nou
            if (prestec.EnPrestec == true)
            {
                Llibre llibre = db.Llibres.Single(Ll => Ll.LlibreId == prestec.LlibreId);
                if (llibre != null)
                {
                    llibre.EnPrestec = false;
                    db.ObjectStateManager.ChangeObjectState(llibre, EntityState.Modified);
                    db.SaveChanges();
                }
            }
            this.db.Prestecs.DeleteObject(prestec);
            this.db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }


        /// <summary>
        /// GET: /Prestec/Retorna/5
        /// Retornem un llibre, en mostrem les dades
        /// </summary>
        /// <parm name="id">identificador del préstec</param>
        public ActionResult Retorna(int id = 0)
        {
            Prestec prestec = db.Prestecs.SingleOrDefault(p => p.PrestecId == id);
            if (prestec == null)
            {
                return HttpNotFound();
            }
            return View("Retorna", prestec);
        }


        /// <summary>
        /// POST: /Prestec/Retorna/5
        /// Retornem un llibre
        /// </summary>
        /// <parm name="id">identificador del préstec</param>
        [HttpPost, ActionName("Retorna")]
        public ActionResult RetornaConfirmed(int id)
        {
            Prestec prestec = db.Prestecs.Single(p => p.PrestecId == id);
            int usuariId = prestec.UsuariId;
            prestec.EnPrestec = false;
            prestec.dataDevolucio = DateTime.Now;
            Llibre llibre = db.Llibres.Single(l => l.LlibreId == prestec.LlibreId);
            llibre.EnPrestec = false;
            db.SaveChanges();

            return RedirectToAction("../Usuari/Details", new { id = prestec.UsuariId });
        }

    }
}