﻿/// <summary>
/// Aquesta és la classe responsable de controlar el tipus d'usuari i les pàgines (vistes) que se'n deriven
/// </summary>
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using WebAdmin.Models;

namespace WebAdmin.Controllers
{
    public class TipusUsuariController : Controller
    {
        private BibliotecaEntities1 db = new BibliotecaEntities1();

        //
        // GET: /TipusUsuari/

        public ActionResult Index()
        {
            return View(db.TipusUsuaris.ToList());
        }

        //
        // GET: /TipusUsuari/Details/5

        public ActionResult Details(int id = 0)
        {
            TipusUsuari tipususuari = db.TipusUsuaris.Single(t => t.TipusUsuariId == id);
            if (tipususuari == null)
            {
                return HttpNotFound();
            }
            return View(tipususuari);
        }

        //
        // GET: /TipusUsuari/Create

        public ActionResult Create()
        {
            return View();
        }

        //
        // POST: /TipusUsuari/Create

        [HttpPost]
        public ActionResult Create(TipusUsuari tipususuari)
        {
            if (ModelState.IsValid)
            {
                db.TipusUsuaris.AddObject(tipususuari);
                db.SaveChanges();
                return RedirectToAction("Index");
            }

            return View(tipususuari);
        }

        //
        // GET: /TipusUsuari/Edit/5

        public ActionResult Edit(int id = 0)
        {
            TipusUsuari tipususuari = db.TipusUsuaris.Single(t => t.TipusUsuariId == id);
            if (tipususuari == null)
            {
                return HttpNotFound();
            }
            return View(tipususuari);
        }

        //
        // POST: /TipusUsuari/Edit/5

        [HttpPost]
        public ActionResult Edit(TipusUsuari tipususuari)
        {
            if (ModelState.IsValid)
            {
                db.TipusUsuaris.Attach(tipususuari);
                db.ObjectStateManager.ChangeObjectState(tipususuari, EntityState.Modified);
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            return View(tipususuari);
        }

        //
        // GET: /TipusUsuari/Delete/5

        public ActionResult Delete(int id = 0)
        {
            var usuaris = from p in db.Usuaris
                          select p;

            usuaris = usuaris.Where(p => p.TipusUsuariId == id);
            if (usuaris.Count() != 0)
            {
                List<String> ErrorList = new List<String>();
                ErrorList.Add("Existeixen Usuaris del tipus a eliminar, siusplau elimina o canvia el tipus dels llibres abans de procedir a l'eliminació de la mateixa");
                foreach (var p in usuaris.ToList())
                {
                    ErrorList.Add("   - UsuariId: " + p.UsuariId.ToString() + " " + p.Nom + " " + p.Cognom + " " + p.Cognom2);
                }
                ViewBag.errorList = ErrorList;
                ViewBag.errorTitle = "Error Eliminant el Tipus d'usuari";
                ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                return View("../Error/Error");
            }
            TipusUsuari tipususuari = db.TipusUsuaris.Single(t => t.TipusUsuariId == id);
            if (tipususuari == null)
            {
                return HttpNotFound();
            }
            return View(tipususuari);
        }

        //
        // POST: /TipusUsuari/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            TipusUsuari tipususuari = db.TipusUsuaris.Single(t => t.TipusUsuariId == id);
            db.TipusUsuaris.DeleteObject(tipususuari);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}