﻿/// <summary>
/// Aquesta és la classe responsable de controlar l'usuari i les pàgines (views) que se'n deriven
/// </summary>

using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using WebAdmin.Models;
using PagedList;
using WebAdmin;
using System.Web.Security;
using WebMatrix;
using WebMatrix.WebData;
using System.Net.Mail;

namespace WebAdmin.Controllers
{
    public class UsuariController : Controller
    {
        private BibliotecaEntities1 db = new BibliotecaEntities1();

        /// <summary>
        /// GET: /Usuari/
        /// Mostrem un llistat d'usuaris
        /// </summary>
        /// <parm name="searchString">No l'utilitzem</param>
        /// <parm name="searchString">No l'utilitzem</param>
        /// <parm name="searchString">Cadena de búsqueda per tal de filtrar els usuaris per cognoms </param>
        /// <parm name="page">Pàgina actual</param>
        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {

            if (Request.HttpMethod == "GET")
            {
                searchString = currentFilter;
            }
            else
            {
                page = 1;
            }
            ViewBag.CurrentFilter = searchString;

            var usuaris = from u in db.Usuaris
                           select u;
            // Filtrem els usuaris segons el critèri de búsqueda ìntroduit per l'usuari (cerca per cognoms)
            usuaris = usuaris.OrderBy(u => u.Cognom);
            if (!String.IsNullOrEmpty(searchString))
            {
                usuaris = usuaris.Where(u => u.Cognom.ToUpper().Contains(searchString.ToUpper())
                                       || u.Cognom2.ToUpper().Contains(searchString.ToUpper()));
            }
            int pageSize = 10;
            int pageNumber = (page ?? 1);
            int total = usuaris.Count();
            if (total == 0)
                ViewBag.IsEmpty = true;
            else
                ViewBag.IsEmpty = false;

            return View(usuaris.ToPagedList(pageNumber, pageSize));
        }


        /// <summary>
        /// GET: /Usuari/Details/5
        /// Mostrem l'informació d'un usuari en particular
        /// </summary>
        /// <parm name="id">Identificador de l'usuari</param>
        public ActionResult Details(int id = 0)
        {
            Usuari usuari = db.Usuaris.SingleOrDefault(u => u.UsuariId == id);
            if (usuari == null)
            {
                return HttpNotFound();
            }

            var prestecsUsuari = from p in db.Prestecs
                   where p.UsuariId == id
                   select p;
            prestecsUsuari = prestecsUsuari.OrderByDescending(l => l.dataPrestec);
            ViewBag.Prestecs = prestecsUsuari;

            return View(usuari);
        }


        /// <summary>
        /// GET: /Usuari/Create
        /// Obrim la view que ens permet crear un usuari
        /// </summary>
        public ActionResult Create()
        {
            ViewBag.TipusUsuariId = new SelectList(db.TipusUsuaris, "TipusUsuariId", "NomTipusUsuari");
            return View();
        }


        /// <summary>
        /// GET: /Usuari/Details/5
        /// Recollim l'informació del formulari de creació d'un nou usuari,
        /// </summary>
        /// <parm name="usuari">dades de l'usuari des del formulari de creació via el model Usuari</param>
        [HttpPost]
        public ActionResult Create(Usuari usuari)
        {
            if (ModelState.IsValid)
            {
                if (usuari.AccesWeb == true)
                {
                    // Intentem registrar l'usuari                       
                    UserProfile currentProfile = null;
                    Usuari usuariMateixEmail = null;
                    try
                    {
                        // Comprovem si ja existeix un usuari amb el mateix email (ha d'esser únic per tal de poder crear els perfils online on l'email és el username)
                        usuariMateixEmail = db.Usuaris.FirstOrDefault(u => u.Email == usuari.Email);
                        if (usuariMateixEmail == null)
                        {
                            // L'email no existex, procedim a crear el perfil
                            String tempPassword = System.Web.Security.Membership.GeneratePassword(10, 2);
                            WebSecurity.CreateUserAndAccount(usuari.Email, tempPassword);
                            currentProfile = db.UserProfiles.FirstOrDefault(u => u.UserName == usuari.Email);
                            // Ara comprovem si existeix un perfil online amb aquest email
                            if (currentProfile != null)
                            {
                                // No existeix, procedim
                                bool emailResult = EnviarEmailBenvinguda(usuari.Email, tempPassword, usuari.Nom + " " + usuari.Cognom);
                                if (!emailResult)
                                {
                                    List<String> ErrorList = new List<String>();
                                    ErrorList.Add("Error enviant l'email" + usuari.Email);
                                    ViewBag.errorList = ErrorList;
                                    ViewBag.errorTitle = "Error al crear l'usuari, no s'ha pogut enviar l'email";
                                    ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                                    return View("../Error/Error");
                                }
                            }
                            else
                            {
                                // L'email ja existeix al perfil online, abortem i reenviem a pàgina d'error
                                List<String> ErrorList = new List<String>();
                                ErrorList.Add("Ja existeix un usuari amb l'email " + usuari.Email);
                                ViewBag.errorList = ErrorList;
                                ViewBag.errorTitle = "Error al crear l'usuari, utilitza un altre email";
                                ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                                return View("../Error/Error");
                            }
                        }
                        else
                        {
                            // Ja existeix un usuari amb aquest email
                            List<String> ErrorList = new List<String>();
                            ErrorList.Add("L'email ja existeix per un usuari registrat");
                            ErrorList.Add("Usuari Registrat:");
                            ErrorList.Add("Id: " + usuariMateixEmail.UsuariId.ToString());
                            ErrorList.Add(usuariMateixEmail.Nom + " " + usuariMateixEmail.Cognom + " " + usuariMateixEmail.Cognom2);
                            ErrorList.Add(usuariMateixEmail.DNI.ToString());

                            ViewBag.errorList = ErrorList;
                            ViewBag.errorTitle = "Error creant el perfil" + usuari.Email;

                            ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                            return View("../Error/Error");
                        }
                    }
                    catch (MembershipCreateUserException ex)
                    {
                        List<String> ErrorList = new List<String>();
                        ErrorList.Add("Error desconegut afegint usuari" + usuari.Email);
                        ViewBag.errorList = ErrorList;
                        ViewBag.errorTitle = "Error al crear l'usuari";
                        ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                        return View("../Error/Error");
                    }
                    usuari.UserId = currentProfile.UserId;
                }
                else
                {
                    Usuari usuariMateixEmail = null;
                    // Comprovem si ja existeix un usuari amb el mateix email (ha d'esser únic per tal de poder crear els perfils online on l'email és el username)
                    usuariMateixEmail = db.Usuaris.FirstOrDefault(u => u.Email == usuari.Email);
                    if (usuariMateixEmail != null)
                    {
                        // L'email ja existeix al perfil online, abortem i reenviem a pàgina d'error
                        List<String> ErrorList = new List<String>();
                        ErrorList.Add("Ja existeix un usuari amb l'email " + usuari.Email);
                        ErrorList.Add("Id: " + usuariMateixEmail.UsuariId.ToString());
                        ErrorList.Add(usuariMateixEmail.Nom + " " + usuariMateixEmail.Cognom + " " + usuariMateixEmail.Cognom2);
                        ErrorList.Add(usuariMateixEmail.DNI.ToString());
                        ViewBag.errorList = ErrorList;
                        ViewBag.errorTitle = "Error al crear l'usuari, utilitza un altre email";
                        ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                        return View("../Error/Error");
                    }
                }
                db.Usuaris.AddObject(usuari);
                db.SaveChanges();
                return RedirectToAction("Index");
            }

            ViewBag.TipusUsuariId = new SelectList(db.TipusUsuaris, "TipusUsuariId", "NomTipusUsuari", usuari.TipusUsuariId);
            return View(usuari);
        }

        //
        // GET: /Usuari/Edit/5

        /// <summary>
        /// GET: /Usuari/Edit/5
        /// Editem l'usuari i obrim la vista que ens permet modificar les dades
        /// </summary>
        /// <parm name="id">l'identificador de l'usuari a editar</param>
        public ActionResult Edit(int id = 0)
        {
            Usuari usuari = db.Usuaris.SingleOrDefault(u => u.UsuariId == id);
            if (usuari == null)
            {
                return HttpNotFound();
            }
            ViewBag.TipusUsuariId = new SelectList(db.TipusUsuaris, "TipusUsuariId", "NomTipusUsuari", usuari.TipusUsuariId);
            return View(usuari);
        }


        /// <summary>
        /// POST: /Usuari/Edit/5
        /// Recollim les modificacions del l'usuari, les verifiquem i les actualitzem
        /// </summary>
        /// <parm name="usuari">dades del tipus usuari des del formulari de edició via el model Usuari</param>
        [HttpPost]
        public ActionResult Edit(Usuari usuari)
        {
            if (ModelState.IsValid)
            {
                // Comprovem que no hem canviat l'email i ja existeix en un altre profile
                
                // Comprovem si ja existeix un usuari amb el mateix email (ha d'esser únic per tal de poder crear els perfils online on l'email és el username)
                var usuariMateixEmail = from a in db.Usuaris
                                        select a;
                usuariMateixEmail = usuariMateixEmail.Where(u => u.Email == usuari.Email);
                usuariMateixEmail = usuariMateixEmail.Where(u => u.UsuariId != usuari.UsuariId);

                if (usuariMateixEmail.Count() >0 )
                {
                    // L'email ja existeix al perfil online, abortem i reenviem a pàgina d'error
                    List<String> ErrorList = new List<String>();
                    ErrorList.Add("Ja existeix un usuari amb l'email " + usuari.Email);
                    ErrorList.Add("Id: " + usuariMateixEmail.First().UsuariId.ToString());
                    ErrorList.Add(usuariMateixEmail.First().Nom + " " + usuariMateixEmail.First().Cognom + " " + usuariMateixEmail.First().Cognom2);
                    ErrorList.Add(usuariMateixEmail.First().DNI.ToString());
                    ViewBag.errorList = ErrorList;
                    ViewBag.errorTitle = "Error a l'actualitzar l'usuari, utilitza un altre email";
                    ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                    return View("../Error/Error");
                }
                // L'usuari no té un perfil online
                if (usuari.UserId == null)
                {
                    // Si l'usuari té accés web aleshores comprovem que l'adreça no existeix
                    if (usuari.AccesWeb == true)
                    {
                        UserProfile profileAmbEmail = null;
                        profileAmbEmail = db.UserProfiles.FirstOrDefault(u => u.UserName == usuari.Email);
                        if (profileAmbEmail == null)
                        {
                            // No existeix un profileweb amb aquesta adreça d'email, procedim a crear el perfil web
                            String tempPassword = System.Web.Security.Membership.GeneratePassword(10, 2);
                            WebSecurity.CreateUserAndAccount(usuari.Email, tempPassword);
                            int onlineId = db.UserProfiles.FirstOrDefault(u => u.UserName == usuari.Email).UserId;
                            usuari.UserId = onlineId;
                            // Enviem email de benvinguda amb informació d'accés a la compte
                            bool emailResult = EnviarEmailBenvinguda(usuari.Email, tempPassword, usuari.Nom + " " + usuari.Cognom);
                            // En cas d'error envian't l'email reenviem a la pàgina d'error.
                            if (!emailResult)
                            {
                                List<String> ErrorList = new List<String>();
                                ErrorList.Add("Error al crear l'usuari, no s'ha pogut enviar l'email");
                                ViewBag.errorList = ErrorList;
                                ViewBag.errorTitle = "Error enviant l'email" + usuari.Email;
                                ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                                return View("../Error/Error");
                            }
                        }
                        // Ja existeix un usuari amb aquest email
                        else
                        {
                            // Busquem qui té l'email
                            Usuari quiTeLemail = null;
                            quiTeLemail = db.Usuaris.FirstOrDefault(u => u.Email == usuari.Email);
                            if (quiTeLemail == null)
                            {
                                // No hem pogut trobar qui té l'email
                                List<String> ErrorList = new List<String>();
                                ErrorList.Add("L'email ja existeix per en un usuari online encara que no el té cap usuari");
                                ViewBag.errorList = ErrorList;
                                ViewBag.errorTitle = "Error creant el perfil online" + usuari.Email;
                                ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                                return View("../Error/Error");
                            }
                            else
                            {
                                // Hem trobat qui té l'email
                                List<String> ErrorList = new List<String>();
                                ErrorList.Add("L'email ja existeix per un usuari registrat");
                                ErrorList.Add("Usuari Registrat:");
                                ErrorList.Add("Id: " + quiTeLemail.UsuariId.ToString());
                                ErrorList.Add(quiTeLemail.Nom + " " + quiTeLemail.Cognom + " " + quiTeLemail.Cognom2);
                                ErrorList.Add(quiTeLemail.DNI.ToString());

                                ViewBag.errorList = ErrorList;
                                ViewBag.errorTitle = "Error creant el perfil online" + usuari.Email;

                                ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                                return View("../Error/Error");
                            }
                        }
                    }
                }
                // L'usuari ja té un perfil online
                else 
                {
                    if (usuari.AccesWeb == false)
                    {
                        // Hem desactivat l'accés web
                        // Eliminem el link entre usuari i usuari online i borrem l'usuari online
                        string webUserName = db.UserProfiles.FirstOrDefault(u => u.UserId == usuari.UserId).UserName;
                        var roles = (SimpleRoleProvider)Roles.Provider;
                        var membership = (SimpleMembershipProvider)Membership.Provider;
                        bool wasDeleted = membership.DeleteAccount(webUserName);
                        wasDeleted = membership.DeleteUser(webUserName, true);
                        usuari.UserId = null;
                    }
                    else
                    {
                        // L'usuari ja és web i existeix un perfile web per a ell
                        UserProfile currentProfile = null;
                        currentProfile = db.UserProfiles.FirstOrDefault(u => u.UserId == usuari.UserId);
                        if (currentProfile == null)
                        {
                            // L'usuari hauria de tenir un perfil online i no el té
                            List<String> ErrorList = new List<String>();
                            ErrorList.Add("L'usuari hauria de tenir un perfil online pero no el té");
                            ViewBag.errorList = ErrorList;
                            ViewBag.errorTitle = "Error canviant usuariOnline";
                            ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                            return View("../Error/Error");
                        }
                        else
                        {
                            if (currentProfile.UserName != usuari.Email)
                            {
                                // L'usuari ha canviat el seu email
                                // Busquem si el nou email el té un altre usuari online
                                Usuari quiTeLemail = null;
                                quiTeLemail = db.Usuaris.FirstOrDefault(u => u.Email == usuari.Email);
                                if (quiTeLemail == null)
                                {
                                    // Cap usuari té l'email, procedim a modificar l'email (username) de l'usuari online
                                    // Eliminem el perfil online i el recreem doncs no podem canviar l'username (email)
                                    var roles = (SimpleRoleProvider)Roles.Provider;
                                    var membership = (SimpleMembershipProvider)Membership.Provider;
                                    bool wasDeleted = membership.DeleteAccount(currentProfile.UserName);
                                    wasDeleted = membership.DeleteUser(currentProfile.UserName, true);
                                    String tempPassword = System.Web.Security.Membership.GeneratePassword(10, 2);
                                    WebSecurity.CreateUserAndAccount(usuari.Email, tempPassword);
                                    bool emailResult = EnviarEmailBenvinguda(usuari.Email, tempPassword, usuari.Nom + " " + usuari.Cognom);
                                    int onlineId = db.UserProfiles.FirstOrDefault(u => u.UserName == usuari.Email).UserId;
                                    usuari.UserId = onlineId;
                                    db.Usuaris.Attach(usuari);
                                    db.ObjectStateManager.ChangeObjectState(usuari, EntityState.Modified);
                                    db.SaveChanges();
                                    return View("../Error/NouPerfil");
                                }
                                else
                                {
                                    // L'Email ja existex, enviem missatge d'error
                                    List<String> ErrorList = new List<String>();
                                    ErrorList.Add("L'email ja existeix per un usuari registrat");
                                    ErrorList.Add("Usuari Registrat:");
                                    ErrorList.Add("Id: " + quiTeLemail.UsuariId.ToString());
                                    ErrorList.Add(quiTeLemail.Nom + " " + quiTeLemail.Cognom + " " + quiTeLemail.Cognom2);
                                    ErrorList.Add(quiTeLemail.DNI.ToString());

                                    ViewBag.errorList = ErrorList;
                                    ViewBag.errorTitle = "Error creant el perfil online" + usuari.Email;

                                    ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                                    return View("../Error/Error");
                                }
                            }
                        }
                    }
                }

                db.Usuaris.Attach(usuari);
                db.ObjectStateManager.ChangeObjectState(usuari, EntityState.Modified);
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            ViewBag.TipusUsuariId = new SelectList(db.TipusUsuaris, "TipusUsuariId", "NomTipusUsuari", usuari.TipusUsuariId);
            return View(usuari);
        }

        /// <summary>
        /// GET: /Usuari/Delete/5
        /// Presentem el formulari d'eliminació d'usuari amb identificador id
        /// </summary>
        /// <parm name="id">identificador de l'usuari a eliminar</param>
        public ActionResult Delete(int id = 0)
        {
            var prestecs = from p in db.Prestecs
                          select p;
            prestecs = prestecs.Include("Llibre");
            prestecs = prestecs.Where(p => p.UsuariId == id);
            if (prestecs.Count() != 0)
            { 
                List<String> ErrorList = new List<String>();
                ErrorList.Add("Existeixen prèstecs associats a l'usuari, siusplau elimina els prèstecs abans de procedir amb l'eliminació de l'usuari");
                foreach (var p in prestecs.ToList())
                {
                    ErrorList.Add("   - PrestedId: " + p.PrestecId.ToString() + " - Llibre: " + p.Llibre.LlibreId.ToString() + " " + p.Llibre.Titol + " " + p.Llibre.ISBN.ToString());
                }
                ViewBag.errorList = ErrorList;
                ViewBag.errorTitle = "Error Eliminant l'usuari";
                ViewBag.Referrer = Request.UrlReferrer.LocalPath;
                return View("../Error/Error");
            }
            Usuari usuari = db.Usuaris.FirstOrDefault(u => u.UsuariId == id);
            if (usuari == null)
            {
                return HttpNotFound();
            }
            return View(usuari);
        }


        /// <summary>
        /// POST: /Usuari/Delete/5
        /// Eliminem l'usuari
        /// </summary>
        /// <parm name="accestipu">identificador de l'usuari a eliminar</param>
        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            Usuari usuari = db.Usuaris.Single(u => u.UsuariId == id);
            if (usuari.UserId != null)
            {
                UserProfile usuariOnline = db.UserProfiles.FirstOrDefault(u => u.UserId == usuari.UserId);
                if (usuariOnline != null)
                {
                    //WebSecurity.Del.DeleteUser(usuariOnline.UserName, true);

                    var roles = (SimpleRoleProvider)Roles.Provider;
                    var membership = (SimpleMembershipProvider)Membership.Provider;
                    bool wasDeleted = membership.DeleteAccount(usuariOnline.UserName);
                    wasDeleted = membership.DeleteUser(usuariOnline.UserName, true);
                }
            }
            db.Usuaris.DeleteObject(usuari);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        /// <summary>
        /// GET: /IndexLlibre/
        /// Presentem les dades de l'usuari definit per l'UsuariId amb una llista de llibres amb el seu nivell d'accés
        /// </summary>
        /// <parm name="UsuariId">Identificador de l'usuarir</param>
        /// <parm name="searchStringISBN">Filtre de llibres per ISBN</param>
        /// <parm name="searchStringTitol">Filtre del llibres per títol</param>
        /// <parm name="AutorId">Filtre de llibres per autor</param>
        /// <parm name="page">Pàgina actual per la paginació</param>
        public ActionResult IndexLlibres(int UsuariId, string searchStringISBN, string searchStringTitol, string AutorId, int? page)
        {
            if (Request.HttpMethod != "GET")
            {
                page = 1;
            }

            ViewBag.CurrentFilterISBN = searchStringISBN;
            ViewBag.CurrentFilterTitol = searchStringTitol;
            ViewBag.CurrentFilterAutor = AutorId;

            Usuari usuari = db.Usuaris.SingleOrDefault(u => u.UsuariId == UsuariId);
            if (usuari == null)
            {
                return HttpNotFound();
            }
            ViewBag.CurrentUser = usuari;

            ViewBag.AutorId = new SelectList(db.Autors, "AutorId", "NomAutor");

            // Creem la llista de llibres amb els permisos d'accés corresponents

            // Seleccionem tots els llibres
            var llibres = from l in db.Llibres
                          select l;
            
            // Seleccionem tots els Tipus d'accés (tipus de llibre - Tipus d'usuari - true)
            var acces = from a in db.AccesTipus
                         select a;

            // Filtrem els tipus d'accés i només deixem aquells que pertanyen al tipus d'usuari de l'usuari actual
            acces = acces.Where(a => a.TipusUsuariId == usuari.TipusUsuariId);

            // Filtrem els tipus d'accés que són true (accés permés)
            acces = acces.Where(a => a.Access == true);
             
            // Filtrem els llibres segons els criteris de búsqueda (si l'usuari n'ha especidicat algún)
            // ISBN
            if (!String.IsNullOrEmpty(searchStringISBN))
            {
                llibres = llibres.Where(l => l.ISBN.ToUpper().Contains(searchStringISBN.ToUpper()));
            }
            // Títol
            if (!String.IsNullOrEmpty(searchStringTitol))
            {
                llibres = llibres.Where(l => l.Titol.ToUpper().Contains(searchStringTitol.ToUpper()));
            }
            // Autor
            int currentAutor;
            if (!String.IsNullOrEmpty(AutorId))
            {
                currentAutor = Convert.ToInt32(AutorId);
                llibres = llibres.Where(l => l.AutorId == currentAutor);
            }

            // Finalment creem  una joint entre els llibres restants (després d'haver aplicat el filtre) i els tipus d'accés pel tipus d'usuari, si no existeix accés definit per un
            // per el tipus d'usuari i el tipus de llibre del llibre actual en deneguem l'accés, per exemple si ell llibre actual és del tipus revista, i l'usuari és del tipus estudiant
            // i no tenim definit un tipus d'accés pels estudiants a les revistes en deneguem l'accés
            var query = from l in llibres
            join a in acces on l.TipusLlibreId equals a.TipusLlibreId into Accessibilitat
            from x in Accessibilitat.DefaultIfEmpty()
            select new
                {
                    LlibreId = l.LlibreId,
                    Access = (x == null ? false : x.Access)
                };

            // Creem una llista d'un tipus especial que hem creat que compren un element tipus llibre i un booleà que defineix si l'usuari actual en té accés
            List<llibreAmbPermisos> llistaFinal = new List<llibreAmbPermisos>();

            // Omplim la llista amb els llibres i l'accés
            llibreAmbPermisos llap;
            foreach (var pt in query.ToList())
            {
                llap = new llibreAmbPermisos();
                llap.Access = pt.Access;
                llap.Llibre = db.Llibres.Single(l => l.LlibreId == pt.LlibreId);
                llistaFinal.Add(llap);
            }

            // Inicialitzem / actualitzem els peràmetre específics de la pagedlist
            int pageSize = 10;
            int pageNumber = (page ?? 1);
            int total = llibres.Count();
            if (total == 0)
                ViewBag.IsEmpty = true;
            else
                ViewBag.IsEmpty = false;
            
            // finalment convertim la llista de llibres amb accés a una pagedlist i cridem la vista 
            return View(llistaFinal.ToPagedList(pageNumber, pageSize));
        }

        /// <summary>
        /// Enviem un email de benvinguda amb les dades d'accés online
        /// </summary>
        /// <parm name="desti">destí del missatge</param>
        /// <parm name="password">Password del nou usuari creat</param>
        /// <parm name="nom">Nom d'usuari del nou usuari creat</param>
        private bool EnviarEmailBenvinguda(string desti, string password, string nom)
        {
            Configuracio configuracio = db.Configuracios.Single(c => c.ConfiguracioId == 1);
            MailMessage email = new MailMessage();
            email.To.Add(new MailAddress(desti));
            email.From = new MailAddress(configuracio.Emailnotificacions);
            email.Subject = "Benvingut a la " + configuracio.NomBiblioteca;
            email.Body = "Hola " + nom + "," + "<br/> Per accedir als serveis web de la nostre biblioteca utilitza <br/> Email: " + desti +"<br/>" + "Password: " + password;
            email.IsBodyHtml = true;
            email.Priority = MailPriority.Normal;

            SmtpClient smtp = new SmtpClient();
            smtp.Host = configuracio.ServidorSMTP;
            //smtp.Host =  "outbound.mailhop.org";
            smtp.Port = (int)configuracio.PortSMTP;
            //smtp.Port = 25;
            smtp.EnableSsl = false;
            smtp.UseDefaultCredentials = false;
            //smtp.Credentials = new System.Net.NetworkCredential("adgarett", "letmein");
            smtp.Credentials = new System.Net.NetworkCredential(configuracio.UsuariSMTP, configuracio.PasswordSMTP);

            try{
                smtp.Send(email);
                email.Dispose();
                return true;
            }
            catch (Exception ex)
            {
                //throw (ex);
                return false;
            }
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}