﻿/// <summary>
/// Aquesta és la classe responsable de controlar els llibres i les cerques de llibre
/// </summary>
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.ServiceModel;
using WebExterna.Models;
using PagedList;
using WebExterna.Filters;

namespace WebExterna.Controllers
{
    [InitializeSimpleMembership]
    public class LlibreController : Controller
    {

        //private BibliotecaEntities1 db = new BibliotecaEntities1();

        /// <summary>
        /// GET: /Llibre/
        /// Mostrem un llistat de llibres
        /// </summary>
        /// <parm name="searchStringISBN">Filtre per ISBN</param>
        /// <parm name="searchStringTitol">Filtre per Titol</param>
        /// <parm name="AutorId">Filtre per Autor</param>
        /// <parm name="EditorialId">Filtre per Editorial</param>
        /// <parm name="CategoriaId">Filtre per Categoria</param>
        /// <parm name="TipusllibreId">Filtre per Tipus de Llibre</param>
        /// <parm name="page">Número de pàgina de la paginació</param>
        public ActionResult Index(string searchStringTitol, string searchStringDescripcio, string AutorId, string EditorialId, string CategoriaId, string TipusLlibreId, int? page)
        {
            BibliotecaServiceWebExterna.BibliotecaServiceClient obj = new BibliotecaServiceWebExterna.BibliotecaServiceClient();
            if (Request.HttpMethod != "GET")
            {
                page = 1;
            }

            // Obtenim un llistat de llibres via el servei WCF
            List<WebExterna.BibliotecaServiceWebExterna.Llibre> elsLlibres = new  List<WebExterna.BibliotecaServiceWebExterna.Llibre>();
            WebExterna.BibliotecaServiceWebExterna.Llibre[] array;
            array = obj.GetLlibres(searchStringTitol, searchStringDescripcio, AutorId, EditorialId, CategoriaId, TipusLlibreId);
            foreach(WebExterna.BibliotecaServiceWebExterna.Llibre llibreActual in array)
                elsLlibres.Add(llibreActual);

            // Obtenim un llistat d'autors via el servei WCF per tal de que l'usuari pugui filtrar per autor
            List<WebExterna.BibliotecaServiceWebExterna.Autor> elsAutors = new List<WebExterna.BibliotecaServiceWebExterna.Autor>();
            WebExterna.BibliotecaServiceWebExterna.Autor[] array2;
            array2 = obj.GetAutors();
            foreach (WebExterna.BibliotecaServiceWebExterna.Autor autorActual in array2)
                elsAutors.Add(autorActual);

            // Ordenem els autors per nom
            List<WebExterna.BibliotecaServiceWebExterna.Autor> elsAutorsOrdenats = elsAutors.OrderBy(o => o.NomAutor).ToList();
            ViewBag.AutorId = new SelectList(elsAutorsOrdenats, "AutorId", "NomAutor");

            // Obtenim un llistat d'editorias via el servei WCF per tal de que l'usuari pugui filtrar per editorial
            List<WebExterna.BibliotecaServiceWebExterna.Editorial> lesEditorials = new List<WebExterna.BibliotecaServiceWebExterna.Editorial>();
            WebExterna.BibliotecaServiceWebExterna.Editorial[] arrayEditorials;
            arrayEditorials = obj.GetEditorials();
            foreach (WebExterna.BibliotecaServiceWebExterna.Editorial editorialActual in arrayEditorials)
                lesEditorials.Add(editorialActual);

            // Ordenem les editorials
            List<WebExterna.BibliotecaServiceWebExterna.Editorial> lesEditorialsOrdenades = lesEditorials.OrderBy(o => o.Nom).ToList();
            ViewBag.EditorialId = new SelectList(lesEditorialsOrdenades, "EditorialId", "Nom");

            // Obtenim un llistat de Tipus de llibre via el servei WCF per tal de que l'usuari pugui filtrar per tipus de llibre
            List<WebExterna.BibliotecaServiceWebExterna.TipusLlibre> elsTipus = new List<WebExterna.BibliotecaServiceWebExterna.TipusLlibre>();
            WebExterna.BibliotecaServiceWebExterna.TipusLlibre[] arrayTipus;
            arrayTipus = obj.GetTipusLlibre();
            foreach (WebExterna.BibliotecaServiceWebExterna.TipusLlibre tipusActual in arrayTipus)
                elsTipus.Add(tipusActual);

            // Ordenem els tipus de llibre
            List<WebExterna.BibliotecaServiceWebExterna.TipusLlibre> elsTipusOrdenats = elsTipus.OrderBy(o => o.Nom).ToList();
            ViewBag.TipusLlibreId = new SelectList(elsTipusOrdenats, "TipusLlibreId", "Nom");

            // Obtenim un llistat de Categories via el servei WCF per tal de que l'usuari pugui filtrar per categoria
            List<WebExterna.BibliotecaServiceWebExterna.Categoria> lesCategories = new List<WebExterna.BibliotecaServiceWebExterna.Categoria>();
            WebExterna.BibliotecaServiceWebExterna.Categoria[] arrayCategories;
            arrayCategories = obj.GetCategories();
            foreach (WebExterna.BibliotecaServiceWebExterna.Categoria categoriaActual in arrayCategories)
                lesCategories.Add(categoriaActual);

            // Ordenem les categories
            List<WebExterna.BibliotecaServiceWebExterna.Categoria> lesCategoriesOrdenades = lesCategories.OrderBy(o => o.Nom).ToList();
            ViewBag.CategoriaId = new SelectList(lesCategoriesOrdenades, "CategoriaId", "Nom");

            ViewBag.CurrentFilterTitol = searchStringTitol;
            ViewBag.CurrentFilterDescripcio = searchStringDescripcio;
            ViewBag.CurrentFilterAutor = AutorId;

            int pageSize = 10;
            int pageNumber = (page ?? 1);
            int total = elsLlibres.Count();
            if (total == 0)
                ViewBag.IsEmpty = true;
            else
                ViewBag.IsEmpty = false;

            return View(elsLlibres.ToPagedList(pageNumber, pageSize));
        }


        /// <summary>
        /// GET: /Llibre/Details/5
        /// Mostrem el llibre
        /// </summary>
        /// <parm name="id">l'identificador del llibre a mostrar</param>
        public ActionResult Details(int id = 0)
        {
            BibliotecaServiceWebExterna.BibliotecaServiceClient obj = new BibliotecaServiceWebExterna.BibliotecaServiceClient();
            WebExterna.BibliotecaServiceWebExterna.Llibre llibre = new WebExterna.BibliotecaServiceWebExterna.Llibre();
            llibre = obj.GetLlibre(id);
            if (llibre == null)
            {
                return HttpNotFound();
            }
            return View(llibre);
        }

    }
}
